package de.duehl.algorithm.backtracking;

/*
 * Copyright 2018 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.util.ArrayList;
import java.util.List;

import de.duehl.basics.datetime.time.StopWatch;
import de.duehl.basics.text.NumberString;

/**
 * Diese Klasse stellt einen Backtracking-Algorithmus dar.
 *
 * @version 1.01     2018-01-02
 * @author Christian Dühl
 */

public class Backtracking<T extends AlgorithmState<Step>, Step extends AlgorithmSolutionStep> {

    /** Anzahl der Backtracking-Schritte (zurück). */
    private int backtrackingCount;

    /** Anzahl der rekursiven Aufrufe der Backtracking-Lösungsfunktion. */
    private int recursiveCount;

    /** Anzahl der versuchten Schritte. */
    private int triedStepsCount;

    /** Stoppuhr für die Laufzeit. */
    private final StopWatch watch;

    /** Anzahl der Schritte die abgearbeitet werden müssen, bevor eine neue Ausgabe erzeugt wird. */
    private int numberOfStepsBetweenMessages;

    /** Gibt an, ob Fortschrittsausgaben erzeugt werden sollen. */
    private boolean inform;

    /** Konstruktor. */
    public Backtracking() {
        backtrackingCount = 0;
        recursiveCount = 0;
        triedStepsCount = 0;

        watch = new StopWatch();
    }

    /**
     * Gibt an, dass der Benutzer nach der übergebenen Anzahl von abgearbeiteten Schritten
     * informiert werden will.
     */
    public void informAfterXSteps(int numberOfStepsBetweenMessages) {
        this.numberOfStepsBetweenMessages = numberOfStepsBetweenMessages;
        inform = true;
    }

    /**
     * Rekursives Vorgehen:
     * FindeLoesung(Zustand)
     * 1. wiederhole, solange es noch neue Teil-Lösungsschritte gibt:
     *    a) wähle einen neuen Teil-Lösungsschritt;
     *    b) falls Wahl lokal gültig ist:
     *         - erweitere Zustand um Wahl;
     *         - falls Zustand vollständig ist
     *               return true; // Lösung gefunden!
     *           anderenfalls
     *                falls FindeLoesung(erweiterter Zustand)
     *                    return true; // Lösung!
     *                anderenfalls
     *                    mache Wahl rückgängig; // Sackgasse (Backtracking)!
     * 2. Da es keinen neuen Teil-Lösungsschritt gibt: return false // Keine Lösung!
     */
    public boolean findSolution(T state) {
        ++recursiveCount;

        boolean solutionStepsAvailable = true;
        List<Step> badSteps = new ArrayList<>();

        while (solutionStepsAvailable) {
            Step step = state.chooseNextStep(badSteps);
            if (state.chooseNextStepWasSuccessfull()) {
                countAndPerhapsInformUser();
                state.forward(step);
                if (state.isVaild()) {
                    if (state.isComplete()) {
                        watch.stop();
                        return true;
                    }
                    else {
                        if (findSolution(state)) {
                            return true;
                        }
                        else {
                            state.backward(step);
                            badSteps.add(step);
                            ++backtrackingCount;
                        }
                    }
                }
                else {
                    state.backward(step);
                    badSteps.add(step);
                    // kein echtes Backtracking, war nur nicht zulässig.
                }
            }
            else {
                solutionStepsAvailable = false;
            }
        }

        return false;
    }

    private void countAndPerhapsInformUser() {
        ++triedStepsCount;

        if (inform && triedStepsCount % numberOfStepsBetweenMessages == 0) {
            System.out.println(
                    NumberString.taupu(triedStepsCount) + " Schritte gemacht - " + watch.getTime());
        }
    }

    /** Gibt nach dem Ende des Backtrackings die Anzahl der Backtracking-Schritte an. */
    public int getBacktrackingCount() {
        return backtrackingCount;
    }

    /** Gibt nach dem Ende des Backtrackings die Anzahl der rekursiven Aufrufe an. */
    public int getRecursiveCount() {
        return recursiveCount;
    }

    /** Gibt die Anzahl der versuchten Schritte an. */
    public int getTriedStepsCount() {
        return triedStepsCount;
    }

    /** Gibt nach dem Ende des Backtrackings die Anzahl der Laufzeit an. */
    public String getRuntime() {
        return watch.getTime();
    }

}
