package de.duehl.math.graph.ged.graph.edge;

import de.duehl.math.graph.ged.graph.GedColor;
import de.duehl.math.graph.ged.pixel.PixelPoint;

/**
 * Diese Klasse steht für eine Kante.
 *
 * @version 1.01     2019-06-17
 * @author Christian Dühl
 */

public class Edge {

    public static final int USE_DEFAULT_WIDTH = -17;
    public static final EdgeStyle DEFAULT_STYLE = EdgeStyle.NORMAL;

    public static final double NO_WEIGHT = Double.NEGATIVE_INFINITY;
    public static final double NO_CAPACITY = Double.NEGATIVE_INFINITY;
    public static final double NO_FLOW = Double.NEGATIVE_INFINITY;

    private static final PixelPoint DEFAULT_LABEL_DISTANCE = new PixelPoint(-20, -20);
    private static final PixelPoint DEFAULT_WEIGHT_DISTANCE = new PixelPoint(20, 20);
    private static final PixelPoint DEFAULT_CAPACITY_DISTANCE = new PixelPoint(-20, 20);
    private static final PixelPoint DEFAULT_FLOW_DISTANCE = new PixelPoint(20, -20);

    /** Index der Startecke. */
    private int startVertexIndex;

    /** Index der Zielecke. */
    private int targetVertexIndex;

    /** Farbe der Kante. */
    private GedColor color;

    /** Stil der Kante (durchgehend, gestrichelt oder gepunktet). */
    private EdgeStyle style;

    /** Breite der Kante. */
    private int width;

    /** Bezeichnung der Kante. */
    private String label;

    /** Abweichung von den Koordinaten der Mitte der Kante zur Beschriftung. */
    private PixelPoint labelDistance;

    /** Farbe der Beschriftung. */
    private GedColor labelColor;

    /** Gewicht der Kante. */
    private double weight;

    /** Abweichung von den Koordinaten der Mitte der Kante zum Gewicht. */
    private PixelPoint weightDistance;

    /** Farbe des Gewichts. */
    private GedColor weightColor;

    /** Kapazität der Kante. */
    private double capacity;

    /** Abweichung von den Koordinaten der Mitte der Kante zur Kapazität. */
    private PixelPoint capacityDistance;

    /** Farbe der Kapazität. */
    private GedColor capacityColor;

    /** Fluß auf dieser Kante. */
    private double flow;

    /** Abweichung von den Koordinaten der Mitte der Kante zum Fluss. */
    private PixelPoint flowDistance;

    /** Farbe des Flusses. */
    private GedColor flowColor;

    /** Konstruktor ohne gesetzten Start- und Ziel-Ecken-Index und mit Defaultfarbe. */
    public Edge() {
        this(-1, -1);
    }

    /**
     * Konstruktor mit Defaultfarbe.
     *
     * @param startVertexIndex
     *            Index der Startecke.
     * @param targetVertexIndex
     *            Index der Zielecke.
     */
    public Edge(int startVertexIndex, int targetVertexIndex) {
        this(startVertexIndex, targetVertexIndex, GedColor.DEFAULT);
    }

    /**
     * Konstruktor.
     *
     * @param startVertexIndex
     *            Index der Startecke.
     * @param targetVertexIndex
     *            Index der Zielecke.
     * @param color
     *            Farbe der Kante.
     */
    public Edge(int startVertexIndex, int targetVertexIndex, GedColor color) {
        this.startVertexIndex = startVertexIndex;
        this.targetVertexIndex = targetVertexIndex;
        this.color = color;
        this.style = DEFAULT_STYLE;
        this.width = USE_DEFAULT_WIDTH;
        this.label = "";
        this.labelColor = GedColor.DEFAULT;
        this.labelDistance = DEFAULT_LABEL_DISTANCE;
        this.weight = NO_WEIGHT;
        this.weightColor = GedColor.DEFAULT;
        this.weightDistance = DEFAULT_WEIGHT_DISTANCE;
        this.capacity = NO_CAPACITY;
        this.capacityDistance = DEFAULT_CAPACITY_DISTANCE;
        this.capacityColor = GedColor.DEFAULT;
        this.flow = NO_FLOW;
        this.flowDistance = DEFAULT_FLOW_DISTANCE;
        this.flowColor = GedColor.DEFAULT;
    }

    /**
     * Erzeugt eine Kopie der übergebenen Kante.
     *
     * @param that
     *            Zu kopierende Kante.
     */
    public Edge(Edge that) {
        copyValuesFrom(that);
    }

    /** Kopiert die Werte aus einer anderen Kante. */
    public void copyValuesFrom(Edge that) {
        this.startVertexIndex = that.getStartVertexIndex();
        this.targetVertexIndex = that.getTargetVertexIndex();
        this.color = new GedColor(that.color);
        this.style = that.getStyle();
        this.width = that.getWidth();
        this.label = that.getLabel();
        this.labelColor = new GedColor(that.getLabelColor());
        this.labelDistance = new PixelPoint(that.getLabelDistance());
        this.weight = that.getWeight();
        this.weightColor = new GedColor(that.getWeightColor());
        this.weightDistance = new PixelPoint(that.getWeightDistance());
        this.capacity = that.getCapacity();
        this.capacityDistance = new PixelPoint(that.getCapacityDistance());
        this.capacityColor = new GedColor(that.getCapacityColor());
        this.flow = that.getFlow();
        this.flowDistance = new PixelPoint(that.getFlowDistance());
        this.flowColor = new GedColor(that.getFlowColor());
    }

    /** Getter für den Index der Startecke. */
    public int getStartVertexIndex() {
        return startVertexIndex;
    }

    /** Setter für den Index der Startecke. */
    public void setStartVertexIndex(int startVertexIndex) {
        this.startVertexIndex = startVertexIndex;
    }

    /** Getter für den Index der Zielecke. */
    public int getTargetVertexIndex() {
        return targetVertexIndex;
    }

    /** Setter für den Index der Zielecke. */
    public void setTargetVertexIndex(int targetVertexIndex) {
        this.targetVertexIndex = targetVertexIndex;
    }

    /** Getter für die Farbe der Kante. */
    public GedColor getColor() {
        return color;
    }

    /** Setter für die Farbe der Kante. */
    public void setColor(GedColor color) {
        this.color = color;
    }

    /** Setzt die Farbe der Kante auf den Default-Wert. */
    public void setColorToDefault() {
        this.color = GedColor.DEFAULT;
    }

    /** Getter für den Stil der Kante (durchgehend, gestrichelt oder gepunktet). */
    public EdgeStyle getStyle() {
        return style;
    }

    /** Setter für den Stil der Kante (durchgehend, gestrichelt oder gepunktet). */
    public void setStyle(EdgeStyle style) {
        this.style = style;
    }

    /** Getter für die Breite der Kante. */
    public int getWidth() {
        return width;
    }

    /** Setter für die Breite der Kante. */
    public void setWidth(int width) {
        this.width = width;
    }

    /** Setzt die Breite der Kante auf Default. */
    public void setWidthToDefault() {
        width = USE_DEFAULT_WIDTH;
    }

    /** Getter für die Kapazität der Kante. */
    public double getCapacity() {
        return capacity;
    }

    /** Setter für die Kapazität der Kante. */
    public void setCapacity(double capacity) {
        this.capacity = capacity;
    }

    /** Getter für den Fluß auf dieser Kante. */
    public double getFlow() {
        return flow;
    }

    /** Setter für den Fluß auf dieser Kante. */
    public void setFlow(double flow) {
        this.flow = flow;
    }

    /** Getter für die Bezeichnung der Kante. */
    public String getLabel() {
        return label;
    }

    /** Setter für die Bezeichnung der Kante. */
    public void setLabel(String label) {
        this.label = label;
    }

    /** Getter für die Abweichung von den Koordinaten der Mitte der Kante zur Beschriftung. */
    public PixelPoint getLabelDistance() {
        return labelDistance;
    }

    /**
     * Getter für die Abweichung von den Koordinaten der Mitte der Kante zur Beschriftung in
     * x-Richtung.
     */
    public int getLabelDistanceX() {
        return labelDistance.getX();
    }

    /**
     * Getter für die Abweichung von den Koordinaten der Mitte der Kante zur Beschriftung in
     * y-Richtung.
     */
    public int getLabelDistanceY() {
        return labelDistance.getY();
    }

    /** Setter für die Abweichung von den Koordinaten der Mitte der Kante zur Beschriftung. */
    public void setLabelDistance(PixelPoint labelDistance) {
        this.labelDistance = labelDistance;
    }

    /**
     * Setter für die Abweichung von den Koordinaten der Mitte der Kante zur Beschriftung in
     * x-Richtung.
     */
    public void setLabelDistanceX(int x) {
        labelDistance.setX(x);
    }

    /**
     * Setter für die Abweichung von den Koordinaten der Mitte der Kante zur Beschriftung in
     * y-Richtung.
     */
    public void setLabelDistanceY(int y) {
        labelDistance.setY(y);
    }

    /** Getter für die Farbe der Beschriftung. */
    public GedColor getLabelColor() {
        return labelColor;
    }

    /** Setter für die Farbe der Beschriftung. */
    public void setLabelColor(GedColor labelColor) {
        this.labelColor = labelColor;
    }

    /** Getter für das Gewicht der Kante. */
    public double getWeight() {
        return weight;
    }

    /** Setter für das Gewicht der Kante. */
    public void setWeight(double weight) {
        this.weight = weight;
    }

    /** Getter für Abweichung von den Koordinaten der Mitte der Kante zum Gewicht. */
    public PixelPoint getWeightDistance() {
        return weightDistance;
    }

    /** Getter für Abweichung von den Koordinaten der Mitte der Kante zum Gewicht in x-Richtung. */
    public int getWeightDistanceX() {
        return weightDistance.getX();
    }

    /** Getter für Abweichung von den Koordinaten der Mitte der Kante zum Gewicht in y-Richtung. */
    public int getWeightDistanceY() {
        return weightDistance.getY();
    }

    /** Setter für Abweichung von den Koordinaten der Mitte der Kante zum Gewicht. */
    public void setWeightDistance(PixelPoint weightDistance) {
        this.weightDistance = weightDistance;
    }

    /** Setter für Abweichung von den Koordinaten der Mitte der Kante zum Gewicht in x-Richtung. */
    public void setWeightDistanceX(int x) {
        weightDistance.setX(x);
    }

    /** Setter für Abweichung von den Koordinaten der Mitte der Kante zum Gewicht in y-Richtung. */
    public void setWeightDistanceY(int y) {
        weightDistance.setY(y);
    }

    /** Getter für die Farbe des Gewichts. */
    public GedColor getWeightColor() {
        return weightColor;
    }

    /** Setter für die Farbe des Gewichts. */
    public void setWeightColor(GedColor weightColor) {
        this.weightColor = weightColor;
    }

    /** Getter für die Abweichung von den Koordinaten der Mitte der Kante zur Kapazität. */
    public PixelPoint getCapacityDistance() {
        return capacityDistance;
    }

    /**
     * Getter für die Abweichung von den Koordinaten der Mitte der Kante zur Kapazität in
     * x-Richtung.
     */
    public int getCapacityDistanceX() {
        return capacityDistance.getX();
    }

    /**
     * Getter für die Abweichung von den Koordinaten der Mitte der Kante zur Kapazität in
     * y-Richtung.
     */
    public int getCapacityDistanceY() {
        return capacityDistance.getY();
    }

    /** Setter für die Abweichung von den Koordinaten der Mitte der Kante zur Kapazität. */
    public void setCapacityDistance(PixelPoint capacityDistance) {
        this.capacityDistance = capacityDistance;
    }

    /**
     * Setter für die Abweichung von den Koordinaten der Mitte der Kante zur Kapazität in
     * x-Richtung.
     */
    public void setCapacityDistanceX(int x) {
        capacityDistance.setX(x);
    }

    /**
     * Setter für die Abweichung von den Koordinaten der Mitte der Kante zur Kapazität in
     * y-Richtung.
     */
    public void setCapacityDistanceY(int y) {
        capacityDistance.setY(y);
    }

    /** Getter für die Farbe der Kapazität. */
    public GedColor getCapacityColor() {
        return capacityColor;
    }

    /** Setter für die Farbe der Kapazität. */
    public void setCapacityColor(GedColor capacityColor) {
        this.capacityColor = capacityColor;
    }

    /** Getter für die Abweichung von den Koordinaten der Mitte der Kante zum Fluss. */
    public PixelPoint getFlowDistance() {
        return flowDistance;
    }

    /** Getter für die Abweichung von den Koordinaten der Mitte der Kante zum Fluss in x-Richtung. */
    public int getFlowDistanceX() {
        return flowDistance.getX();
    }

    /** Getter für die Abweichung von den Koordinaten der Mitte der Kante zum Fluss in y-Richtung. */
    public int getFlowDistanceY() {
        return flowDistance.getY();
    }

    /** Setter für die Abweichung von den Koordinaten der Mitte der Kante zum Fluss. */
    public void setFlowDistance(PixelPoint flowDistance) {
        this.flowDistance = flowDistance;
    }

    /** Setter für die Abweichung von den Koordinaten der Mitte der Kante zum Fluss in x-Richtung. */
    public void setFlowDistanceX(int x) {
        flowDistance.setX(x);
    }

    /** Setter für die Abweichung von den Koordinaten der Mitte der Kante zum Fluss in y-Richtung. */
    public void setFlowDistanceY(int y) {
        flowDistance.setY(y);
    }

    /** Getter für die Farbe des Flusses. */
    public GedColor getFlowColor() {
        return flowColor;
    }

    /** Setter für die Farbe des Flusses. */
    public void setFlowColor(GedColor flowColor) {
        this.flowColor = flowColor;
    }

    public boolean hasDefaultStyle() {
        return style == DEFAULT_STYLE;
    }

    public boolean hasDefaultColor() {
        return GedColor.isDefaultColor(color);
    }

    public boolean hasDefaultWidth() {
        return width == USE_DEFAULT_WIDTH;
    }

    public boolean hasLabel() {
        return !label.isEmpty();
    }

    public boolean hasDefaultLabelColor() {
        return GedColor.isDefaultColor(labelColor);
    }

    public boolean hasDefaultWeight() {
        return weight == NO_WEIGHT;
    }

    public boolean hasWeight() {
        return !hasDefaultWeight();
    }

    public boolean hasDefaultWeightColor() {
        return GedColor.isDefaultColor(weightColor);
    }

    public boolean hasCapacity() {
        return !hasDefaultCapacity();
    }

    public boolean hasDefaultCapacity() {
        return capacity == NO_CAPACITY;
    }

    public boolean hasDefaultCapacityColor() {
        return GedColor.isDefaultColor(capacityColor);
    }

    public boolean hasFlow() {
        return !hasDefaultFlow();
    }

    public boolean hasDefaultFlow() {
        return flow == NO_FLOW;
    }

    public boolean hasDefaultFlowColor() {
        return GedColor.isDefaultColor(flowColor);
    }

    public boolean hasDefaultLabelDistance() {
        return labelDistance.equals(DEFAULT_LABEL_DISTANCE);
    }

    public boolean hasDefaultWeightDistance() {
        return labelDistance.equals(DEFAULT_WEIGHT_DISTANCE);
    }

    public boolean hasDefaultCapacityDistance() {
        return labelDistance.equals(DEFAULT_CAPACITY_DISTANCE);
    }

    public boolean hasDefaultFlowDistance() {
        return labelDistance.equals(DEFAULT_FLOW_DISTANCE);
    }

    @Override
    public String toString() {
        return "Edge [\n"
                + "    startVertexIndex=" + startVertexIndex + ",\n"
                + "    targetVertexIndex=" + targetVertexIndex + ",\n"
                + "    color="+ color + ",\n"
                + "    style=" + style + ",\n"
                + "    width=" + width + ",\n"
                + "    label=" + label + ",\n"
                + "    labelDistance=" + labelDistance + ",\n"
                + "    labelColor=" + labelColor + ",\n"
                + "    weight=" + weight + ",\n"
                + "    weightDistance=" + weightDistance + ",\n"
                + "    weightColor=" + weightColor + ",\n"
                + "    capacity=" + capacity + ",\n"
                + "    capacityDistance=" + capacityDistance + ",\n"
                + "    capacityColor=" + capacityColor + ",\n"
                + "    flow=" + flow + ",\n"
                + "    flowDistance=" + flowDistance + ",\n"
                + "    flowColor=" + flowColor + "\n"
                + "]";
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        long temp;
        temp = Double.doubleToLongBits(capacity);
        result = prime * result + (int) (temp ^ (temp >>> 32));
        result = prime * result + ((capacityColor == null) ? 0 : capacityColor.hashCode());
        result = prime * result + ((capacityDistance == null) ? 0 : capacityDistance.hashCode());
        result = prime * result + ((color == null) ? 0 : color.hashCode());
        temp = Double.doubleToLongBits(flow);
        result = prime * result + (int) (temp ^ (temp >>> 32));
        result = prime * result + ((flowColor == null) ? 0 : flowColor.hashCode());
        result = prime * result + ((flowDistance == null) ? 0 : flowDistance.hashCode());
        result = prime * result + ((label == null) ? 0 : label.hashCode());
        result = prime * result + ((labelColor == null) ? 0 : labelColor.hashCode());
        result = prime * result + ((labelDistance == null) ? 0 : labelDistance.hashCode());
        result = prime * result + startVertexIndex;
        result = prime * result + ((style == null) ? 0 : style.hashCode());
        result = prime * result + targetVertexIndex;
        temp = Double.doubleToLongBits(weight);
        result = prime * result + (int) (temp ^ (temp >>> 32));
        result = prime * result + ((weightColor == null) ? 0 : weightColor.hashCode());
        result = prime * result + ((weightDistance == null) ? 0 : weightDistance.hashCode());
        result = prime * result + width;
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        Edge other = (Edge) obj;
        if (Double.doubleToLongBits(capacity) != Double.doubleToLongBits(other.capacity))
            return false;
        if (capacityColor == null) {
            if (other.capacityColor != null)
                return false;
        }
        else if (!capacityColor.equals(other.capacityColor))
            return false;
        if (capacityDistance == null) {
            if (other.capacityDistance != null)
                return false;
        }
        else if (!capacityDistance.equals(other.capacityDistance))
            return false;
        if (color == null) {
            if (other.color != null)
                return false;
        }
        else if (!color.equals(other.color))
            return false;
        if (Double.doubleToLongBits(flow) != Double.doubleToLongBits(other.flow))
            return false;
        if (flowColor == null) {
            if (other.flowColor != null)
                return false;
        }
        else if (!flowColor.equals(other.flowColor))
            return false;
        if (flowDistance == null) {
            if (other.flowDistance != null)
                return false;
        }
        else if (!flowDistance.equals(other.flowDistance))
            return false;
        if (label == null) {
            if (other.label != null)
                return false;
        }
        else if (!label.equals(other.label))
            return false;
        if (labelColor == null) {
            if (other.labelColor != null)
                return false;
        }
        else if (!labelColor.equals(other.labelColor))
            return false;
        if (labelDistance == null) {
            if (other.labelDistance != null)
                return false;
        }
        else if (!labelDistance.equals(other.labelDistance))
            return false;
        if (startVertexIndex != other.startVertexIndex)
            return false;
        if (style != other.style)
            return false;
        if (targetVertexIndex != other.targetVertexIndex)
            return false;
        if (Double.doubleToLongBits(weight) != Double.doubleToLongBits(other.weight))
            return false;
        if (weightColor == null) {
            if (other.weightColor != null)
                return false;
        }
        else if (!weightColor.equals(other.weightColor))
            return false;
        if (weightDistance == null) {
            if (other.weightDistance != null)
                return false;
        }
        else if (!weightDistance.equals(other.weightDistance))
            return false;
        if (width != other.width)
            return false;
        return true;
    }

}
