package de.duehl.math.graph.ged.graph.io.ged4;

import static de.duehl.math.graph.ged.graph.io.ged4.Ged4LoadAndSaveConstants.*;

import java.util.List;

import de.duehl.basics.exceptions.WrongSourceFormatException;
import de.duehl.math.graph.ged.graph.GedColor;
import de.duehl.math.graph.ged.graph.Graph;
import de.duehl.math.graph.ged.graph.edge.Edge;
import de.duehl.math.graph.ged.graph.edge.EdgeStyle;

/**
 * Diese Klasse lädt eine Kante eines Graphen aus dem .ged4-Format.
 *
 * @version 1.01     2019-06-17
 * @author Christian Dühl
 */

public class Ged4EdgeLoader {

    /** Am Semikolon aufgeteilte Teile einer Zeile mit Informationen zu einer Kante. */
    private final List<String> partsOfEdgeLine;

    /** Der Graph in den hier geladen wird. */
    private final Graph graph;

    /** Gerade behandelte Zeile für Fehlerausgaben. */
    private final String actualLine;

    /** Nummer der gerade behandelten Zeile für Fehlerausgaben. */
    private final int actualLineNumber;

    /** Eine Kante die hier geladen wird. */
    private Edge edge;

    /**
     * Konstruktor.
     *
     * @param partsOfEdgeLine
     *            Am Semikolon aufgeteilte Teile einer Zeile mit Informationen zu einer Kante.
     * @param graph
     *            Der Graph in den hier geladen wird.
     * @param actualLine
     *            Gerade behandelte Zeile für Fehlerausgaben.
     * @param actualLineNumber
     *            Nummer der gerade behandelten Zeile für Fehlerausgaben.
     */
    public Ged4EdgeLoader(List<String> partsOfEdgeLine, Graph graph, String actualLine,
            int actualLineNumber) {
        this.partsOfEdgeLine = partsOfEdgeLine;
        this.graph = graph;
        this.actualLine = actualLine;
        this.actualLineNumber = actualLineNumber;
    }

    public void load() {
        edge = new Edge();

        for (String part : partsOfEdgeLine) {
            loadOnePart(part);
        }

        if (edge.getStartVertexIndex() == -1) {
            throw new WrongSourceFormatException("Die Kante\n\t" + edge
                    + "\that keinen Startindex. In Zeile Nummer " + actualLineNumber + "\n\t"
                    + actualLine + "\tim ged4-Format!");
        }
        if (edge.getTargetVertexIndex() == -1) {
            throw new WrongSourceFormatException("Die Kante\n\t" + edge
                    + "\that keinen Endindex. In Zeile Nummer " + actualLineNumber + "\n\t"
                    + actualLine + "\tim ged4-Format!");
        }

        graph.addEdge(edge);
    }

    private void loadOnePart(String part) {
        Variable variable = Ged4Helper.determineVariable(part, actualLine, actualLineNumber);
        String key = variable.getName();
        String value = variable.getValue();

        switch (key) {
            case EDGE_START_VERTEX :
                int startVertexIndex = Integer.parseInt(value);
                edge.setStartVertexIndex(startVertexIndex);
                break;
            case EDGE_END_VERTEX :
                int targetVertexIndex = Integer.parseInt(value);
                edge.setTargetVertexIndex(targetVertexIndex);
                break;
            case EDGE_COLOR :
                edge.setColor(new GedColor(value));
                break;
            case EDGE_STYLE :
                switch (value) {
                    case EDGE_STYLE_NORMAL :
                        edge.setStyle(EdgeStyle.NORMAL);
                        break;
                    case EDGE_STYLE_DASHED :
                        edge.setStyle(EdgeStyle.DASHED);
                        break;
                    case EDGE_STYLE_DOTTED :
                        edge.setStyle(EdgeStyle.DOTTED);
                        break;
                    default:
                        throw new WrongSourceFormatException(""
                                + "Unbekannter Stil '" + value
                                + "' in Teil '" + part
                                + "' in Zeile Nummer " + actualLineNumber + "\n\t" + actualLine
                                + "\tim ged4-Format!");
                }
                break;
            case EDGE_WIDTH :
                edge.setWidth(Integer.parseInt(Ged4Helper.doubleToInt(value)));
                break;
            case EDGE_LABEL :
                edge.setLabel(value);
                break;
            case EDGE_LABEL_X_DISTANCE :
                edge.getLabelDistance().setX(Integer.parseInt(Ged4Helper.doubleToInt(value)));
                break;
            case EDGE_LABEL_Y_DISTANCE :
                edge.getLabelDistance().setY(Integer.parseInt(Ged4Helper.doubleToInt(value)));
                break;
            case EDGE_LABEL_COLOR :
                edge.setLabelColor(new GedColor(value));
                break;
            case EDGE_WEIGHT :
                edge.setWeight(Double.parseDouble(value));
                break;
            case EDGE_WEIGHT_X_DISTANCE :
                edge.getWeightDistance().setX(Integer.parseInt(Ged4Helper.doubleToInt(value)));
                break;
            case EDGE_WEIGHT_Y_DISTANCE :
                edge.getWeightDistance().setY(Integer.parseInt(Ged4Helper.doubleToInt(value)));
                break;
            case EDGE_WEIGHT_COLOR :
                edge.setWeightColor(new GedColor(value));
                break;
            case EDGE_CAPACITY :
                edge.setCapacity(Double.parseDouble(value));
                break;
            case EDGE_CAPACITY_X_DISTANCE :
                edge.getCapacityDistance().setX(Integer.parseInt(Ged4Helper.doubleToInt(value)));
                break;
            case EDGE_CAPACITY_Y_DISTANCE :
                edge.getCapacityDistance().setY(Integer.parseInt(Ged4Helper.doubleToInt(value)));
                break;
            case EDGE_CAPACITY_COLOR :
                edge.setCapacityColor(new GedColor(value));
                break;
            case EDGE_FLOW :
                edge.setFlow(Double.parseDouble(value));
                break;
            case EDGE_FLOW_X_DISTANCE :
                edge.getFlowDistance().setX(Integer.parseInt(Ged4Helper.doubleToInt(value)));
                break;
            case EDGE_FLOW_Y_DISTANCE :
                edge.getFlowDistance().setY(Integer.parseInt(Ged4Helper.doubleToInt(value)));
                break;
            case EDGE_FLOW_COLOR :
                edge.setFlowColor(new GedColor(value));
                break;
        }
    }

}
