package de.duehl.math.graph.ged.graph.io.ged4;

import static de.duehl.math.graph.ged.graph.io.ged4.Ged4LoadAndSaveConstants.*;

import java.util.List;

import de.duehl.basics.exceptions.WrongSourceFormatException;
import de.duehl.basics.text.Text;
import de.duehl.basics.text.handling.BooleanHandling;
import de.duehl.math.graph.ged.graph.GedColor;
import de.duehl.math.graph.ged.graph.Graph;
import de.duehl.math.graph.ged.graph.MetaData;

/**
 * Diese Klasse lädt einen Graphen aus dem .ged4-Format.
 *
 * @version 1.01     2019-06-17
 * @author Christian Dühl
 */

public class Ged4FormatLoader {

    /** Der Graph in den hier geladen wird. */
    private final Graph graph;

    /** Enthält weitere Informationen zum Graphen. */
    private final MetaData meta; // zur Bequemlichkeit.

    /** Gerade behandelte Zeile für Fehlerausgaben. */
    private String actualLine;

    /** Nummer der gerade behandelten Zeile für Fehlerausgaben. */
    private int actualLineNumber;

    /**
     * Konstruktor.
     * @param graph Der Graph in den hier geladen wird.
     */
    public Ged4FormatLoader(Graph graph) {
        this.graph = graph;
        meta = graph.getMeta();
    }

    /**
     * Lädt den Graphen aus dem .ged4-Format und ersetzt den aktuellen Graphen
     * durch den geladenen.
     *
     * @param text
     *            Eingelesene Datei als ein String.
     * @throws WrongSourceFormatException
     *             Falls das Format nicht verstanden wird.
     */
    public void loadFromSaveString(String text) {
        graph.clearGraph();
        actualLineNumber = 0;

        List<String> lines = Text.splitByLineBreaks(text);

        for (String line : lines) {
            workOnLine(line);
        }
    }

    private void workOnLine(String line) {
        actualLine = line;
        ++actualLineNumber;

        List<String> parts = Text.splitBySemicolon(line);
        if (parts.size() >= 1) {
            String firtsPart = parts.get(0);
            parts.remove(0);
            if (META_LINE_START.equals(firtsPart)) {
                workOnMetaParts(parts);
            }
            else if (VERTEX_LINE_START.equals(firtsPart)) {
                workOnVertexParts(parts);
            }
            else if (EDGE_LINE_START.equals(firtsPart)) {
                workOnEdgeParts(parts);
            }
            else if (!line.isEmpty()) {
                throw new WrongSourceFormatException("Unbekannter " + "Zeilenanfang '" + firtsPart
                        + "' in Zeile\n\t" + line + "\tim ged4-Format!");
            }
        }
    }

    private void workOnMetaParts(List<String> parts) {
        for (String part : parts) {
            workOnMetaPart(part);
        }
    }

    private void workOnMetaPart(String part) {
        Variable variable = Ged4Helper.determineVariable(part, actualLine, actualLineNumber);
        String key = variable.getName();
        String value = variable.getValue();
        switch (key) {
            case META_DIRECTED :
                meta.setDirected(BooleanHandling.stringToBoolean(value));
                break;
            case META_DEFAULT_VERTEX_COLOR :
                meta.setDefaultVertexColor(new GedColor(value));
                break;
            case META_DEFAULT_EDGE_COLOR :
                meta.setDefaultEdgeColor(new GedColor(value));
                break;
            case META_DEFAULT_RADIUS :
                meta.setDefaultRadius(Integer.parseInt(Ged4Helper.doubleToInt(value)));
                break;
            case META_BACKGROUND_COLOR :
                meta.setBackgroundColor(new GedColor(value));
                break;
            case META_SHOW_GRID :
                switch (value) {
                    case "true" : // wurde früher so gespeichert ...
                        meta.setShowGrid(true);
                        break;
                    case "false" :
                        meta.setShowGrid(false);
                        break;
                     default:
                         meta.setShowGrid(BooleanHandling.stringToBoolean(value));
                         break;
                }
                break;
            case META_GRID_DISTANCE :
                meta.setGridDistance(Integer.parseInt(Ged4Helper.doubleToInt(value)));
                break;
            case META_SNAP_IN_GRID :
                switch (value) {
                    case "true" : // wurde früher so gespeichert ...
                        meta.setSnapInGrid(true);
                        break;
                    case "false" :
                        meta.setSnapInGrid(false);
                        break;
                     default:
                         meta.setSnapInGrid(BooleanHandling.stringToBoolean(value));
                         break;
                }
                break;
            case META_GRID_COLOR :
                meta.setGridColor(new GedColor(value));
                break;
            case META_BACKGROUND_PICTURE :
                meta.setBackgroundPicture(value);
                break;
            case META_DEFAULT_EDGE_WIDTH :
                meta.setDefaultEdgeWidth(Integer.parseInt(Ged4Helper.doubleToInt(value)));
                break;
        }
    }

    private void workOnVertexParts(List<String> partsOfVertexLine) {
        Ged4VertexLoader vertexLoader = new Ged4VertexLoader(partsOfVertexLine, graph, actualLine,
                actualLineNumber);
        vertexLoader.load();
    }

    private void workOnEdgeParts(List<String> partsOfEdgeLine) {
        Ged4EdgeLoader edgeLoader = new Ged4EdgeLoader(partsOfEdgeLine, graph, actualLine,
                actualLineNumber);
        edgeLoader.load();
    }

}
