package de.duehl.math.graph.ged.graph.io.ged4;

import static de.duehl.math.graph.ged.graph.io.ged4.Ged4LoadAndSaveConstants.*;

import java.util.List;

import de.duehl.basics.exceptions.WrongSourceFormatException;
import de.duehl.math.graph.ged.graph.GedColor;
import de.duehl.math.graph.ged.graph.Graph;
import de.duehl.math.graph.ged.graph.vertex.Vertex;

/**
 * Diese Klasse lädt eine Ecke eines Graphen aus dem .ged4-Format.
 *
 * @version 1.01     2019-06-17
 * @author Christian Dühl
 */

public class Ged4VertexLoader {

    /** Am Semikolon aufgeteilte Teile einer Zeile mit Informationen zu einer Ecke. */
    private final List<String> partsOfVertexLine;

    /** Der Graph in den hier geladen wird. */
    private final Graph graph;

    /** Gerade behandelte Zeile für Fehlerausgaben. */
    private final String actualLine;

    /** Nummer der gerade behandelten Zeile für Fehlerausgaben. */
    private final int actualLineNumber;

    /** Eine Ecke die hier geladen wird. */
    private Vertex vertex;

    /**
     * Index die die Ecke im gespeicherten Graphen hatte - zu Kontrolle, ob sie diese auch im
     * geladenen Graphen wieder bekommt.
     */
    private int hasToHaveVertexNumber;

    /**
     * Konstruktor.
     *
     * @param partsOfVertexLine
     *            Am Semikolon aufgeteilte Teile einer Zeile mit Informationen zu einer Ecke.
     * @param graph
     *            Der Graph in den hier geladen wird.
     * @param actualLine
     *            Gerade behandelte Zeile für Fehlerausgaben.
     * @param actualLineNumber
     *            Nummer der gerade behandelten Zeile für Fehlerausgaben.
     */
    public Ged4VertexLoader(List<String> partsOfVertexLine, Graph graph, String actualLine,
            int actualLineNumber) {
        this.partsOfVertexLine = partsOfVertexLine;
        this.graph = graph;
        this.actualLine = actualLine;
        this.actualLineNumber = actualLineNumber;
    }

    public void load() {
        vertex = new Vertex();
        hasToHaveVertexNumber = -1;

        for (String part : partsOfVertexLine) {
            loadOnePart(part);
        }

        if (-1 == hasToHaveVertexNumber) {
            throw new WrongSourceFormatException("Die Ecke\n\t" + vertex
                    + "\that keine Nummer abgespeichert. "
                    + "In Zeile Nummer " + actualLineNumber + "\n\t"
                    + actualLine + "\tim ged4-Format!");
        }

        graph.addVertex(vertex);

        if (vertex.getIndex() != hasToHaveVertexNumber) {
            throw new WrongSourceFormatException("Die Ecke\n\t" + vertex
                    + "\that nicht die vorgesehene Nummer " + hasToHaveVertexNumber
                    + " zugewiesen bekommen. In Zeile Nummer " + actualLineNumber + "\n\t"
                    + actualLine + "\tim ged4-Format!");
        }
    }

    private void loadOnePart(String part) {
        Variable variable = Ged4Helper.determineVariable(part, actualLine, actualLineNumber);
        String key = variable.getName();
        String value = variable.getValue();

        switch (key) {
            case VERTEX_NUMBER :
                hasToHaveVertexNumber = Integer.parseInt(value);
                break;
            case VERTEX_X :
                vertex.setX(Integer.parseInt(Ged4Helper.doubleToInt(value)));
                break;
            case VERTEX_Y :
                vertex.setY(Integer.parseInt(Ged4Helper.doubleToInt(value)));
                break;
            case VERTEX_COLOR :
                vertex.setColor(new GedColor(value));
                break;
            case VERTEX_RADIUS :
                vertex.setRadius(Integer.parseInt(Ged4Helper.doubleToInt(value)));
                break;
            case VERTEX_LABEL :
                vertex.setLabel(value);
                break;
            case VERTEX_LABEL_X_DISTANCE :
                vertex.getLabelDistance().setX(Integer.parseInt(Ged4Helper.doubleToInt(value)));
                break;
            case VERTEX_LABEL_Y_DISTANCE :
                vertex.getLabelDistance().setY(Integer.parseInt(Ged4Helper.doubleToInt(value)));
                break;
            case VERTEX_LABEL_COLOR :
                vertex.setLabelColor(new GedColor(value));
                break;
            case VERTEX_WEIGHT :
                vertex.setWeight(Double.parseDouble(value));
                break;
            case VERTEX_WEIGHT_X_DISTANCE :
                vertex.getWeightDistance().setX(Integer.parseInt(Ged4Helper.doubleToInt(value)));
                break;
            case VERTEX_WEIGHT_Y_DISTANCE :
                vertex.getWeightDistance().setY(Integer.parseInt(Ged4Helper.doubleToInt(value)));
                break;
            case VERTEX_WEIGHT_COLOR :
                vertex.setWeightColor(new GedColor(value));
                break;
        }
    }

}
