package de.duehl.math.graph.ged.ui.dialogs;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridLayout;
import java.text.NumberFormat;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JFormattedTextField;
import javax.swing.JPanel;

import de.duehl.basics.collections.CollectionsHelper;
import de.duehl.basics.text.NumberString;
import de.duehl.math.graph.ged.pixel.PixelPoint;
import de.duehl.math.graph.ged.ui.GuiElements;
import de.duehl.swing.ui.GuiTools;
import de.duehl.swing.ui.dialogs.base.ModalDialogBase;

/**
 * Diese Klasse öffnet einen kleinen Dialog zum Wählen einer Größe aus fest vorgegebenen Werten.
 *
 * @version 1.02     2019-07-02
 * @author Christian Dühl
 */

public class SizeChooser extends ModalDialogBase {

    public static final int NO_VALUE = -1;

    /** Koordinaten, an denen der Dialog angezeigt werden soll. */
    private final PixelPoint coords;

    /** Enthält die grafischen Elemente. */
    private final GuiElements elements;

    /** Hier ausgewählte neue Größe. */
    private int size;


    /**
     * Konstruktor.
     *
     * @param oldSize
     *            Alte Größe.
     * @param description
     *            Beschreibung der Einheit, dessen Größe geändert werden soll ("Radius").
     * @param coords
     *            Koordinaten, an denen der Dialog angezeigt werden soll.
     * @param elements
     *            Enthält die grafischen Elemente.
     */
    public SizeChooser(int oldSize, PixelPoint coords, String description, GuiElements elements) {
        super(elements.getFrameLocation(), elements.getProgramImage(),
                "Größe für " + description + " wählen (alter Wert ist " + oldSize + "):");
        addEscapeBehaviour();

        size = NO_VALUE;
        this.coords = coords;
        this.elements = elements;

        fillDialog();
    }

    @Override
    protected void populateDialog() {
        add(createNumberButtonsPart(), BorderLayout.CENTER);
        add(createEnterValuePart(), BorderLayout.SOUTH);

        setLocation();
    }

    private void setLocation() {
        MouseClickLocationSetter setter = new MouseClickLocationSetter(this, coords, elements);
        setter.setLocation();
    }

    private Component createNumberButtonsPart() {
        JPanel panel = new JPanel();
        panel.setLayout(new GridLayout(5, 4));

        List<Integer> sizes = CollectionsHelper.buildListFrom(
                1,  6,  11,  16,  35,
                2,  7,  12,  18,  40,
                3,  8,  13,  20,  50,
                4,  9,  14,  25,  75,
                5, 10,  15,  30, 100
                );

        for (int newSize : sizes) {
            JButton button = new JButton(Integer.toString(newSize));
            button.addActionListener(e -> valueEntered(newSize));
            panel.add(button);
        }

        return panel;
    }

    private Component createEnterValuePart() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());

        NumberFormat fmt = NumberFormat.getInstance();
        fmt.setMaximumFractionDigits(0); // no fractions - integer only
        fmt.setMinimumFractionDigits(0); // as above
        JFormattedTextField valueTextField = new JFormattedTextField(fmt);
        GuiTools.setEditFieldColors(valueTextField);
        valueTextField.addActionListener(e -> valueEntered(valueTextField.getText()));

        //valueTextField.setBackground(Color.WHITE); // TODO Methoden aus GuiTools nehmen?
        //valueTextField.setForeground(foregroundColor);
        panel.add(valueTextField, BorderLayout.CENTER);

        // TODO: Button mit OK daneben?

        return panel;
    }

    private void valueEntered(String newSizeAsString) {
        valueEntered(NumberString.parseIntIgnore(newSizeAsString, NO_VALUE));
    }

    private void valueEntered(int newSize) {
        size = newSize;
        closeDialog();
    }

    /**
     * Getter für die gewählte Größe.
     *
     * Bei Abbruch des Dialogs wird NO_VALUE, also -1 zurückgegeben.
     */
    public int getSize() {
        return size;
    }

}
