package de.duehl.math.graph.ged.graph.io;

import static de.duehl.math.graph.ged.graph.io.Ged3LoadAndSaveConstants.*;

import de.duehl.basics.text.xml.generation.XmlBuilder;
import de.duehl.math.graph.ged.graph.Graph;
import de.duehl.math.graph.ged.graph.MetaData;
import de.duehl.math.graph.ged.graph.edge.AdjacenceEntry;
import de.duehl.math.graph.ged.graph.edge.AdjacenceLists;
import de.duehl.math.graph.ged.graph.edge.EdgeStyle;
import de.duehl.math.graph.ged.graph.edge.AdjacenceList;
import de.duehl.math.graph.ged.graph.vertex.Vertex;
import de.duehl.math.graph.ged.graph.vertex.Vertices;

/**
 * Diese Klasse erzeugt eine Darstellung des Graphen in ged3-Format zum Speichern in einer Datei.
 *
 * @version 1.01     2019-05-21
 * @author Christian Dühl
 */

public class Ged3FormatSaver {

    /** Eckenmenge des Graphen. */
    private final Vertices vertices;

    /** Kantenmenge des Graphen in Form von Adjazenzlisten. */
    private final AdjacenceLists adjacenceLists;

    /** Enthält weitere Informationen zum Graphen. */
    private MetaData meta;

    /** Erstellt die zu speichernde XML-Datei. */
    private XmlBuilder builder;

    /**
     * Konstruktor.
     *
     * @param graph
     *            Der zu speichernde Graph.
     */
    public Ged3FormatSaver(Graph graph) {
        this.meta = graph.getMeta();
        this.vertices = graph.getVertices();
        this.adjacenceLists = graph.getAdjacenceLists();
    }

    /** Gibt den Graphen als String gespeichert im ged3-Format zurück. */
    public String createSaveString() {
        builder = new XmlBuilder();

        builder.appendOpeningTag(GRAPH);
        appendGraph();
        builder.appendClosingTag(GRAPH);

        String saveString = builder.toString();
        return saveString.replace(">default<", ">undef<");
    }

    private void appendGraph() {
        appendMetaData();
        appendVertices();
        appendEdges();
    }

    private void appendMetaData() {
        builder
                .appendInTag(META_VERTEX_NUMBER, vertices.size())
                .appendInTag(META_EDGE_NUMBER, adjacenceLists.getNumberOfEdges())
                .appendInTag(META_DEFAULT_VERTEX_COLOR,
                        meta.getDefaultVertexColor().toBigColorString())
                .appendInTag(META_DEFAULT_EDGE_COLOR, meta.getDefaultEdgeColor().toBigColorString())
                .appendInTag(META_DEFAULT_RADIUS, meta.getDefaultRadius())
                .appendInTag(META_BACKGROUND_COLOR, meta.getBackgroundColor().toBigColorString())
                .appendInTag(META_SHOW_GRID, meta.isShowGrid() ? "1" : EDGE_TYPE_NORMAL)
                .appendInTag(META_GRID_DISTANCE, meta.getGridDistance())
                .appendInTag(META_SNAP_IN_GRID, meta.isSnapInGrid() ? "1" : EDGE_TYPE_NORMAL)
                .appendInTag(META_GRID_COLOR, meta.getGridColor().toBigColorString())
                .appendInTag(META_BACKGROUND_PICTURE, meta.getBackgroundPicture())
                .appendInTag(META_DEFAULT_EDGE_WIDTH, meta.getDefaultEdgeWidth())
                .appendInTag(META_FILENAME, "") // TODO ...
                ;
    }

    private void appendVertices() {
        builder.appendOpeningTag(VERTICES);
        for (Vertex vertex : vertices) {
            appendVertex(vertex);
        }
        builder.appendClosingTag(VERTICES);
    }

    private void appendVertex(Vertex vertex) {
        builder
                .appendOpeningTag(VERTEX)
                .appendInTag(VERTEX_NUMBER, vertex.getIndex())
                .appendInTag(VERTEX_X, vertex.getX())
                .appendInTag(VERTEX_Y, vertex.getY())
                .appendInTag(VERTEX_COLOR, vertex.getColor().toBigColorString())
                .appendInTag(VERTEX_RADIUS, defaultRadius(vertex))
                .appendInTag(VERTEX_TITLE, vertex.getLabel())
                .appendInTag(VERTEX_WEIGHT, defaultWeight(vertex))
                .appendInTag(VERTEX_TITLE_POSITION, POSITION_NW) // da hier ja keine Zahlen, sondern
                                                          // "NW"
                // und sowas erwartet wird.
                .appendInTag(VERTEX_WEIGHT_POSITION, POSITION_SO) // da hier ja keine Zahlen, sondern
                                                           // "NW"
                // und sowas erwartet wird.
                .appendInTag(VERTEX_TITLE_COLOR, vertex.getLabelColor().toBigColorString())
                .appendInTag(VERTEX_WEIGHT_COLOR, vertex.getWeightColor().toBigColorString())
                .appendClosingTag(VERTEX)                    ;
    }

    private String defaultRadius(Vertex vertex) {
        if (vertex.hasRadius()) {
            return Integer.toString(vertex.getRadius());
        }
        else {
            return "default";
        }
    }

    private String defaultWeight(Vertex vertex) {
        if (vertex.hasWeight()) {
            return Double.toString(vertex.getWeight());
        }
        else {
            return "default";
        }
    }

    private void appendEdges() {
        builder.appendOpeningTag(EDGES);

        int count = 0;
        for (int startIndex=0; startIndex < adjacenceLists.size(); ++startIndex) {
            AdjacenceList list = adjacenceLists.get(startIndex);
            for (AdjacenceEntry entry : list) {
                appendEdge(count, startIndex, entry);
                ++count;
            }
        }

        builder.appendClosingTag(EDGES);
    }

    private void appendEdge(int edgeNumber, int startIndex, AdjacenceEntry entry) {
        builder
                .appendOpeningTag(EDGE)
                .appendInTag(EDGE_NUMBER, edgeNumber)
                .appendInTag(EDGE_START_VERTEX, startIndex)
                .appendInTag(EDGE_END_VERTEX, entry.getNeighbourIndex())
                .appendInTag(EDGE_COLOR, entry.getColor().toBigColorString())
                // TODO: 'form' und dabei berücksichtigen, dass es Gegenkanten
                //       gibt und wie man die dann als eine speichert.
                //       Die Form fehlt hier! -> EDGE_FORM

                .appendInTag(EDGE_TYPE, determineEdgeType(entry.getStyle()))

                .appendInTag(EDGE_WIDTH, entry.getWidth())
                .appendInTag(EDGE_TITLE, entry.getLabel())
                .appendInTag(EDGE_WEIGHT, entry.getWeight())
                .appendInTag(EDGE_TITLE_POSITION, POSITION_NW) // da hier ja keine Zahlen,
                                                               // sondern "NW" und sowas
                                                               // erwartet wird.
                .appendInTag(EDGE_WEIGHT_POSITION, POSITION_SO) // da hier ja keine Zahlen,
                                                                // sondern "NW" und sowas
                                                                // erwartet wird.
                .appendInTag(EDGE_TITLE_COLOR, entry.getLabelColor().toBigColorString())
                .appendInTag(EDGE_WEIGHT_COLOR, entry.getWeightColor().toBigColorString())
                /* Das gehörte in Perl nicht zum Format ged3 !
                .appendInTag(EDGE_CAPACITY, entry.getCapacity())
                .appendInTag(EDGE_CAPACITY_POSITION, POSITION_NO) // da hier ja keine Zahlen,
                                                           // sondern "NW" und sowas
                                                           // erwartet wird.
                .appendInTag(EDGE_CAPACITY_COLOR, entry.getCapacityColor().toBigColorString())
                .appendInTag(EDGE_FLOW, entry.getFlow())
                .appendInTag(EDGE_FLOW_POSITION, POSITION_SW) // da hier ja keine Zahlen,
                                                              // sondern
                                                              // "NW" und sowas erwartet
                                                              // wird.
                .appendInTag(EDGE_FLOW_COLOR, entry.getFlowColor().toBigColorString())
                */
                .appendClosingTag(EDGE);
    }

    private String determineEdgeType(EdgeStyle style) {
        switch (style) {
            default:
            case NORMAL:
                return EDGE_TYPE_NORMAL;
            case DASHED:
                return EDGE_TYPE_GESTRICHELT;
            case DOTTED:
                return EDGE_TYPE_GEPUNKTET;
        }
    }

}
