package de.duehl.math.graph.ged.graph.vertex;

import de.duehl.math.graph.ged.graph.GedColor;
import de.duehl.math.graph.ged.pixel.PixelPoint;

/**
 * Diese Klasse steht für eine Ecke eines Graphen.
 *
 * @version 1.01     2019-05-27
 * @author Christian Dühl
 */

public class Vertex {

    /*
     * Da man bei double im Gegensatz zur Farbe (Color) keine Werte erzeugen
     * kann, die auf die Verwendung der Defaultwerte aus den Metaeigenschaften
     * des Graphen hinweisen, definieren wir hier solche:
     *
     * Wenn die Farbe der Ecke den Wert Color.DEFAULT hat, bedeutet dies, dass
     * die Eckenfarbe aus den Metadaten des Graphen verwendet wird.
     *
     * Wenn der Radius der Ecke den Wert USE_DEFAULT_RADIUS hat, dann bedeutet
     * dies, dass der Radius aus den Metadaten des Graphen verwendet wird.
     *
     * Wenn die Farbe der Beschriftung der Ecke den Wert Color.DEFAULT hat,
     * dann wird die Farbe der Ecke benutzt (bzw. die Farbe aus den Metadaten
     * des Graphen, falls keine eigenständige Eckenfarbe benutzt wird).
     *
     * Wenn das Gewicht der Ecke den Wert NO_WEIGHT hat, dann bedeutet dies,
     * dass das Gewicht nicht angezeigt wird.
     *
     * Wenn die Farbe des Gewichts der Ecke den Wert Color.DEFAULT hat, dann
     * wird die Farbe der Ecke benutzt (bzw. die Farbe aus den Metadaten des
     * Graphen, falls keine eigenständige Eckenfarbe benutzt wird.
     */
    public static final int USE_DEFAULT_RADIUS = -16;
    public static final double NO_WEIGHT = Double.NEGATIVE_INFINITY;

    /** Index der Ecke - wird vom Graphen verwaltet. */
    private int index;

    /** Koordinaten der Ecke */
    private PixelPoint coordinates;

    /** Radius der Ecke. */
    private int radius;

    /** Farbe der Ecke. */
    private GedColor color;

    /** Beschriftung der Ecke. */
    private String label;

    /** Abweichung von den Koordinaten der Ecke zur Beschriftung */
    private PixelPoint labelDistance;

    /** Farbe der Beschriftung. */
    private GedColor labelColor;

    /** Gewicht der Ecke. */
    private double weight;

    /** Abweichung von den Koordinaten der Ecke zum Gewicht. */
    private PixelPoint weightDistance;

    /** Farbe des Gewichts. */
    private GedColor weightColor;

    /**
     * Konstruktor für eine neue Ecke an den Koordinaten 0.0 , 0.0 und mit dem Defaultradius und
     * der Defaultfarbe.
     */
    public Vertex() {
        this(new PixelPoint(), USE_DEFAULT_RADIUS, GedColor.DEFAULT);
    }

    /**
     * Konstruktor.
     *
     * @param coordinates
     *            Koordinaten der Ecke.
     * @param radius
     *            Radius der Ecke.
     * @param color
     *            Farbe der Ecke.
     */
    public Vertex(PixelPoint coordinates, int radius, GedColor color) {
        this.index = -1;
        this.coordinates = new PixelPoint(coordinates);
        this.radius = radius;
        this.color = color;
        this.label = "";
        this.labelDistance = new PixelPoint();
        this.labelColor = new GedColor(color);
        this.weight = NO_WEIGHT;
        this.weightDistance = new PixelPoint();
        this.weightColor = new GedColor(color);
    }

    /**
     * Erzeugt eine Kopie der übergebenen Ecke.
     *
     * @param vertex
     *            zu kopierende Ecke
     */
    public Vertex(Vertex vertex) {
        copyValuesFrom(vertex);
    }

    /** Kopiert die Werte aus einer anderen Ecke. */
    public void copyValuesFrom(Vertex vertex) {
        this.index = vertex.index;
        this.coordinates = new PixelPoint(vertex.coordinates);
        this.radius = vertex.radius;
        this.color = new GedColor(vertex.color);
        this.label = vertex.label;
        this.labelDistance = new PixelPoint(vertex.labelDistance);
        this.labelColor = new GedColor(vertex.labelColor);
        this.weight = vertex.weight;
        this.weightDistance = new PixelPoint(vertex.weightDistance);
        this.weightColor = new GedColor(vertex.getWeightColor());
    }

    /** Getter für den Index der Ecke. */
    public int getIndex() {
        return index;
    }

    /** Setter für den Index der Ecke. */
    public void setIndex(int index) {
        this.index = index;
    }

    /** Getter für die Koordinaten der Ecke. */
    public PixelPoint getCoordinates() {
        return coordinates;
    }

    /** Getter für die X-Koordinate der Ecke. */
    public int getX() {
        return getCoordinates().getX();
    }

    /** Getter für die Y-Koordinate der Ecke. */
    public int getY() {
        return getCoordinates().getY();
    }

    /** Setter für die Koordinaten der Ecke. */
    public void setCoordinates(PixelPoint coordinates) {
        this.coordinates = coordinates;
    }

    /** Setter für die X-Koordinate der Ecke. */
    public void setX(int x) {
        coordinates.setX(x);
    }

    /** Setter für die Y-Koordinate der Ecke. */
    public void setY(int y) {
        coordinates.setY(y);
    }

    /** Getter für den Radius der Ecke. */
    public int getRadius() {
        return radius;
    }

    /** Setter für den Radius der Ecke. */
    public void setRadius(int radius) {
        this.radius = radius;
    }

    /** Setzt den Radius der Ecke auf default. */
    public void setRadiusToDefault() {
        setRadius(USE_DEFAULT_RADIUS);
    }

    /** Getter für die Farbe der Ecke. */
    public GedColor getColor() {
        return color;
    }

    /** Setter für die Farbe der Ecke. */
    public void setColor(GedColor color) {
        this.color = color;
    }

    /** Getter für die Beschriftung der Ecke. */
    public String getLabel() {
        return label;
    }

    /** Setter für die Beschriftung der Ecke. */
    public void setLabel(String label) {
        this.label = label;
    }

    /** Getter für die Abweichung von den Koordinaten der Ecke zur Beschriftung. */
    public PixelPoint getLabelDistance() {
        return labelDistance;
    }

    /** Getter für die Abweichung von den Koordinaten der Ecke zur Beschriftung in x-Richtung. */
    public int getLabelDistanceX() {
        return labelDistance.getX();
    }

    /** Getter für die Abweichung von den Koordinaten der Ecke zur Beschriftung in y-Richtung. */
    public int getLabelDistanceY() {
        return labelDistance.getY();
    }

    /** Setter für die Abweichung von den Koordinaten der Ecke zur Beschriftung. */
    public void setLabelDistance(PixelPoint labelDistance) {
        this.labelDistance = labelDistance;
    }

    /** Setter für die Abweichung von den Koordinaten der Ecke zur Beschriftung in x-Richtung. */
    public void setLabelDistanceX(int x) {
        labelDistance.setX(x);
    }

    /** Setter für die Abweichung von den Koordinaten der Ecke zur Beschriftung in y-Richtung. */
    public void setLabelDistanceY(int y) {
        labelDistance.setY(y);
    }

    /** Getter für die Farbe der Beschriftung. */
    public GedColor getLabelColor() {
        return labelColor;
    }

    /** Setter für die Farbe der Beschriftung. */
    public void setLabelColor(GedColor labelColor) {
        this.labelColor = labelColor;
    }

    /** Getter für das Gewicht der Ecke. */
    public double getWeight() {
        return weight;
    }

    /** Setter für das Gewicht der Ecke. */
    public void setWeight(double weight) {
        this.weight = weight;
    }

    /** Getter für die Abweichung von den Koordinaten der Ecke zum Gewicht. */
    public PixelPoint getWeightDistance() {
        return weightDistance;
    }

    /** Getter für die Abweichung von den Koordinaten der Ecke zum Gewicht in x-Richtung. */
    public int getWeightDistanceX() {
        return weightDistance.getX();
    }

    /** Getter für die Abweichung von den Koordinaten der Ecke zum Gewicht in y-Richtung. */
    public int getWeightDistanceY() {
        return weightDistance.getY();
    }

    /** Setter für die Abweichung von den Koordinaten der Ecke zum Gewicht. */
    public void setWeightDistance(PixelPoint weightDistance) {
        this.weightDistance = weightDistance;
    }

    /** Setter für die Abweichung von den Koordinaten der Ecke zum Gewicht in x-Richtung. */
    public void setWeightDistanceX(int x) {
        weightDistance.setX(x);
    }

    /** Setter für die Abweichung von den Koordinaten der Ecke zum Gewicht in y-Richtung. */
    public void setWeightDistanceY(int y) {
        weightDistance.setY(y);
    }

    /** Getter für die Farbe des Gewichts. */
    public GedColor getWeightColor() {
        return weightColor;
    }

    /** Setter für die Farbe des Gewichts. */
    public void setWeightColor(GedColor weightColor) {
        this.weightColor = weightColor;
    }

    public boolean hasRadius() {
        return !hasDefaultRadius();
    }

    public boolean hasDefaultRadius() {
        return radius == USE_DEFAULT_RADIUS;
    }

    public boolean hasLabel() {
        return !hasDefaultLabel();
    }

    public boolean hasDefaultLabel() {
        return label.isEmpty();
    }

    public boolean hasWeight() {
        return !hasDefaultWeight();
    }

    public boolean hasDefaultWeight() {
        return weight == Vertex.NO_WEIGHT;
    }

    public boolean hasDefaultColor() {
        return GedColor.isDefaultColor(color);
    }

    public boolean hasDefaultLabelColor() {
        return GedColor.isDefaultColor(labelColor);
    }

    public boolean hasDefaultWeightColor() {
        return GedColor.isDefaultColor(weightColor);
    }

    @Override
    public String toString() {
        return "Vertex [\n"
                + "    index=" + index + ",\n"
                + "    coordinates=" + coordinates + ",\n"
                + "    radius=" + radius + ",\n"
                + "    color=" + color + ",\n"
                + "    label=" + label + ",\n"
                + "    labelDistance=" + labelDistance + ",\n"
                + "    labelColor=" + labelColor + ",\n"
                + "    weight=" + weight + ",\n"
                + "    weightDistance=" + weightDistance + ",\n"
                + "    weightColor=" + weightColor + "\n"
                + "]";
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((color == null) ? 0 : color.hashCode());
        result = prime * result + ((coordinates == null) ? 0 : coordinates.hashCode());
        result = prime * result + index;
        result = prime * result + ((label == null) ? 0 : label.hashCode());
        result = prime * result + ((labelColor == null) ? 0 : labelColor.hashCode());
        result = prime * result + ((labelDistance == null) ? 0 : labelDistance.hashCode());
        result = prime * result + radius;
        long temp;
        temp = Double.doubleToLongBits(weight);
        result = prime * result + (int) (temp ^ (temp >>> 32));
        result = prime * result + ((weightColor == null) ? 0 : weightColor.hashCode());
        result = prime * result + ((weightDistance == null) ? 0 : weightDistance.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        Vertex other = (Vertex) obj;
        if (color == null) {
            if (other.color != null)
                return false;
        }
        else if (!color.equals(other.color))
            return false;
        if (coordinates == null) {
            if (other.coordinates != null)
                return false;
        }
        else if (!coordinates.equals(other.coordinates))
            return false;
        if (index != other.index)
            return false;
        if (label == null) {
            if (other.label != null)
                return false;
        }
        else if (!label.equals(other.label))
            return false;
        if (labelColor == null) {
            if (other.labelColor != null)
                return false;
        }
        else if (!labelColor.equals(other.labelColor))
            return false;
        if (labelDistance == null) {
            if (other.labelDistance != null)
                return false;
        }
        else if (!labelDistance.equals(other.labelDistance))
            return false;
        if (radius != other.radius)
            return false;
        if (Double.doubleToLongBits(weight) != Double.doubleToLongBits(other.weight))
            return false;
        if (weightColor == null) {
            if (other.weightColor != null)
                return false;
        }
        else if (!weightColor.equals(other.weightColor))
            return false;
        if (weightDistance == null) {
            if (other.weightDistance != null)
                return false;
        }
        else if (!weightDistance.equals(other.weightDistance))
            return false;
        return true;
    }

    public boolean equalsIgnoreIndex(Vertex obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        Vertex other = (Vertex) obj;
        if (color == null) {
            if (other.color != null)
                return false;
        }
        else if (!color.equals(other.color))
            return false;
        if (coordinates == null) {
            if (other.coordinates != null)
                return false;
        }
        else if (!coordinates.equals(other.coordinates))
            return false;
        if (label == null) {
            if (other.label != null)
                return false;
        }
        else if (!label.equals(other.label))
            return false;
        if (labelColor == null) {
            if (other.labelColor != null)
                return false;
        }
        else if (!labelColor.equals(other.labelColor))
            return false;
        if (labelDistance == null) {
            if (other.labelDistance != null)
                return false;
        }
        else if (!labelDistance.equals(other.labelDistance))
            return false;
        if (radius != other.radius)
            return false;
        if (Double.doubleToLongBits(weight) != Double.doubleToLongBits(other.weight))
            return false;
        if (weightColor == null) {
            if (other.weightColor != null)
                return false;
        }
        else if (!weightColor.equals(other.weightColor))
            return false;
        if (weightDistance == null) {
            if (other.weightDistance != null)
                return false;
        }
        else if (!weightDistance.equals(other.weightDistance))
            return false;
        return true;
    }

}
