#------------------------------------------------------------------------------
# Programm         : H i s t o r i e . p m
#------------------------------------------------------------------------------
# Autor            : Christian Dühl
# Erstellt         : 10. Februar 2004
# Zuletzt geändert : 18. Januar 2005
# Aufgabe          : Klasse zur Verwaltung mehrerer Versionen von beliebigen
#                    Dingen, um Rückgängig und Wiederholen Funktionalitäten
#                    verwenden zu können.
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
# Globale private Datei-Variablen:
#------------------------------------------------------------------------------
my $Debug  = 0;



#------------------------------------------------------------------------------
# Klasse           : H i s t o r i e
# abgeleitet von   : Basis
#------------------------------------------------------------------------------
# Autor            : Christian Dühl
# Beschreibung     : Eine Historien-Verwaltungs-Klasse.
# Attribute        : max              - maximale Anzahl gemerkter Dinge
#                    _anz             - aktuelle Anzahl gemerkter Dinge
#                    _akt             - aktueller Index innerhalb der zu
#                                       merkenden Dinge.
#                    _hist            - Speicher für Dinge.
#                    _status          - Status in dem sich das Historien-
#                                       objekt befindet.
# Klassen-Methoden : new              - Erstellt ein Objekt dieser Klasse.
# Objekt-Methoden  : clone            - Erzeugt ein geclontes Objekt (eine
#                                       Kopie, die referenzierte Memberhash-
#                                       werte richtig kopiert und nicht nur
#                                       eine Kopie der Referenz anlegt.
#                    max              - Getter / Setter
#                    _anz             - Getter / Setter
#                    _akt             - Getter / Setter
#                    _hist            - Getter / Setter
#                    _status          - Getter / Setter
#                    _hist_eintrag    - Getter / Setter für einen Historie-
#                                       Eintrag.
#                    _hist_loesch_    - Entfernt den ersten Eintrag aus dem
#                      ersten_eintrag   Array und löscht diesen (mit interner
#                                       Methode clear).
#                    _hist_loesch_    - Entfernt den Eintrag an der angegebenen
#                             eintrag   Stelle aus dem Array und löscht diesen
#                                       (mit interner Methode clear).
#                    objekt_eintragen - Trägt ein Objekt in die Historie ein.
#                    rueckgaengig     - holt das Element vor dem aktuellen
#                    wiederholung     - holt das Element nach dem aktuellen
#                    _geaendert       - Informiert über Änderung, alle Elemente
#                                       nach dem aktuellen werden gelöscht.
#                    _konsistent      - Überprüft, ob die Daten konsistent
#                                       sind.
#                    _fehler          - Verstirbt mit genauer Inhaltsbeschrei-
#                                       bung.
#
# Genaue Beschreibung von anzahl, max und aktuell.
# max: Anzahl der maximal im Array erlaubten Einträge.
# anz: Anzahl der momentan verwendeten Einträge.
# akt: Nummer des letzten Objekts in der Vergangenheit.
#      Beispiel: sind 2 Einträge im Array und beide liegen in der Zukunft,
#                d.h. man kann sie mit wiederholen wiederherstellen, dann
#                ist aktuell 0, da es keinen letzten "alten" Eintrag gibt.
#                Ist ein Eintrag alt und einer neu, so ist aktuell 1.
#                Sind beide Einträge alt, so ist aktuell 2.
#------------------------------------------------------------------------------
# 0.0.1 - 10.02.2004 - CD - erstellt
#------------------------------------------------------------------------------
package Graph::GED::Historie;

use strict;
use warnings;

use Data::Dumper;

#------------------------------------------------------------------------------
# Globale öffentliche Klassen-Variablen:
#------------------------------------------------------------------------------
use base qw/Graph::GED::Basis/;


#------------------------------------------------------------------------------
# Globale private Klassen-Variablen:
#------------------------------------------------------------------------------
# Stati für State-Machine:
use constant NORMAL            => 0;
use constant NORMAL_ENDE       => 1;
use constant NACH_RUECK        => 2;
use constant NACH_RUECK_ANFANG => 3;
use constant NACH_WIEDER       => 4;
use constant NACH_WIEDER_ENDE  => 5;

my %Stati = (
             NORMAL           , 'NORMAL'           ,
             NORMAL_ENDE      , 'NORMAL_ENDE'      ,
             NACH_RUECK       , 'NACH_RUECK'       ,
             NACH_RUECK_ANFANG, 'NACH_RUECK_ANFANG',
             NACH_WIEDER      , 'NACH_WIEDER'      ,
             NACH_WIEDER_ENDE , 'NACH_WIEDER_ENDE' ,
            );



#------------------------------------------------------------------------------
# Erklärung der State-Machine:
#------------------------------------------------------------------------------
# Status | Beschreibung | Array | Rückgängig | Wiederholen  | Objekt einfügen
#--------+--------------+-------+------------+--------------+------------------
# NORMAL | normales     | akt ==| falls      | %            | 1) akt- Objekt
#        | Arbeiten     |   anz |   anz > 0  |              | -> akt+1
#        | Array nicht  | anz < | 1) liefere |              | 2) ++akt
#        |        voll  |   max | Objekt an  |              | 3) ++anz
#        |              | Akt.  | Stelle akt |              | 4) falls anz=max
#        |              | Zust. | 2) akt.Ob. |              | -> NORMAL_ENDE
#        |              | nicht | -> akt+1   |              |
#        |              | im    | 3) --akt   |              |
#        |              | Array | 4) ++anz   |              |
#        |              |       | 5) Falls   |              |
#        |              |       | akt=0 ->   |              |
#        |              |       | NACH_RUECK |              |
#        |              |       | _ANFANG    |              |
#        |              |       | sonst ->   |              |
#        |              |       | NACH_RUECK |              |
#--------+--------------+-------+------------+--------------+------------------
# NORMAL | normales     | akt== | 1) liefere | %            | 1) lösche ältes-
#  _ENDE | Arbeiten     |   anz | Objekt an  |              | tes Element
#        | Array voll   | anz== | Stelle akt |              | 2) verschiebe
#        |              |   max | 2) lösche  |              | alle Elemente
#        |              | Akt.  | ältestes   |              | nach unten
#        |              | Zust. | Objekt     |              | 3) akt. Objekt
#        |              | nicht | 3) ver-    |              | -> akt
#        |              | im    | schiebe    |              | (anz=akt=max)
#        |              | Array | alle Ele-  |              |
#        |              |       | mente nach |              |
#        |              |       | unten      |              |
#        |              |       | 4) akt.Ob. |              |
#        |              |       | -> akt     |              |
#        |              |       | anz=akt=max|              |
#        |              |       | 5) ->      |              |
#        |              |       | NACH_RUECK |              |
#--------+--------------+-------+------------+--------------+------------------
# NACH_  | nach Rück-   | akt < | 1) liefere | 1) liefere   | 1) lösche alle
#  RUECK | gängig machen|   anz | Objekt an  | Objekt an    | Einträge von
#        |              | anz <=| Stelle akt | Stelle akt+1 | akt+1 bis anz
#        |              |   max | 2) --akt   | 2) ++akt     | 2) anz->akt
#        |              | Akt.  | 3) Falls   | 3) falls     | 3) falls anz=max
#        |              | Zust. | akt=0 ->   | akt=anz ->   | -> NORMAL_ENDE,
#        |              | im    | NACH_RUECK | NACH_WIEDER_ | sonst -> NORMAL
#        |              | Array | _ANFAMG    | ENDE, sonst  |
#        |              | auf   |            | -> NACH_     |
#        |              | Platz |            | WIEDER       |
#        |              | akt   |            |              |
#--------+--------------+-------+------------+--------------+------------------
# NACH_  | nach Rück-   | akt=0 | %          | 1) liefere   | 1) lösche alle
# RUECK_ | gängig machen| anz>0 |            | Objekt an    | Einträge von
# ANFANG | ganz am An-  | anz<  |            | Stelle akt+1 | akt+1 bis anz
#        | fang         |   max |            | 2) ++akt     | 2) anz->akt
#        |              | Akt.  |            | 3) falls     | 3) -> NORMAL
#        |              | Zust. |            | akt=anz ->   |
#        |              | im    |            | NACH_WIEDER_ |
#        |              | Array |            | ENDE, sonst  |
#        |              | auf   |            | -> NACH_     |
#        |              | Platz |            | WIEDER       |
#        |              | akt   |            |              |
#--------+--------------+-------+------------+--------------+------------------
# NACH_  | nach Wieder- | akt>0 | 1) liefere | 1) liefere   | 1) lösche alle
# WIEDER | holung       | Akt.  | Objekt an  | Objekt an    | Einträge von
#        |              | Zust. | Stelle akt | Stelle akt+1 | akt+1 bis anz
#        |              | im    | 2) --akt   | 2) ++akt     | 2) anz->akt
#        |              | Array | 3) falls   | 3) falls     | 3) falls anz=max
#        |              | auf   | akt=0 ->   | akt=anz ->   | -> NORMAL_ENDE,
#        |              | Platz | NACH_RUECK | NACH_WIEDER_ | sonst -> NORMAL
#        |              | akt   | _ANFANG,   | ENDE         |
#        |              |       | sonst ->   |              |
#        |              |       | NACH_RUECK |              |
#--------+--------------+-------+------------+--------------+------------------
# NACH_  | nach Wieder- | akt=  | 1) liefere | %            | 1) -> NORMAL_ENDE
# WIEDER | holung ganz  | anz   | Objekt an  |              |
# _ENDE  | am Ende      | anz<= | Stelle akt |              |
#        |              | max   | 2) --akt   |              |
#        |              | Akt.  | 3) falls   |              |
#        |              | Zust. | akt=0 ->   |              |
#        |              | im    | NACH_RUECK |              |
#        |              | Array | _ANFANG,   |              |
#        |              | auf   | sonst ->   |              |
#        |              | Platz | NACH_RUECK |              |
#        |              | akt   |            |              |
#--------+--------------+-------+------------+--------------+------------------
#------------------------------------------------------------------------------


sub new {
    #--------------------------------------------------------------------------
    # Methode      : n e w
    # Klasse       : Graph::GED::Historie
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Erzeugt eine neue Historie-Objekt.
    # Parameter    : Klasse
    # Rückgabewert : Das geblesste neue Objekt.
    #--------------------------------------------------------------------------
    # 0.0.1 - 10.02.2004 - CD - erstellt
    #--------------------------------------------------------------------------

    my $self  = shift;
    my $class = (ref $self) || $self;

    #--------------------------------------------------------------------------
    # Ausgabe der Aufrufparameter:
    #--------------------------------------------------------------------------
    print "Graph::GED::Historie::new\n",
          "\tself  = '$self'\n",
          "\tclass = '$class'\n",
          if $Debug > 1;


    $self = $class->SUPER::new();


    #--------------------------------------------------------------------------
    # Nun setzen wir noch ein paar Attribute:
    #--------------------------------------------------------------------------
    $self->max(1024);#3);#20);
    $self->_anz(0);
    $self->_akt(0);
    $self->_hist([]);
    $self->_status(NORMAL);


    #--------------------------------------------------------------------------
    # ... und geben das erzeugte Objekt zurück:
    #--------------------------------------------------------------------------
    return $self;

} # sub Graph::GED::Historie::new


sub clone {
    #--------------------------------------------------------------------------
    # Methode      : c l o n e
    # Klasse       : Graph::GED::Historie
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Erzeugt ein geclontes Objekt (eine Kopie, die referen-
    #                zierte Memberhashwerte richtig kopiert und nicht nur eine
    #                Kopie der Referenz anlegt.
    # Parameter    : Ev. ein von einer abgeleiteten Klasse nach unten durchge-
    #                reichtes Objekt, das wir wiederum durchreichen sollen und
    #                anschließend seine in dieser Klasse hinzugefügten Kon-
    #                tainer kopieren sollen.
    # Rückgabewert : Das geclonte Objekt.
    #--------------------------------------------------------------------------
    # 0.0.1 - 12.02.2004 - CD - erstellt
    #--------------------------------------------------------------------------

    my $self = shift;
    my $ursp = shift;
    my $clone;

    print "Graph::GED::Historie::Clone\n" if $Debug;

    #--------------------------------------------------------------------------
    # Bekommen wir von "oben" ein Objekt?
    #--------------------------------------------------------------------------
    if (defined $ursp) {
        $clone = $ursp;
    }
    #--------------------------------------------------------------------------
    # Falls nicht erzeugen wir das Objekt und kopieren unsere Daten flach:
    #--------------------------------------------------------------------------
    else {
        $clone = $self->new();
        %$clone = %$self;
    }

    #--------------------------------------------------------------------------
    # Geerbte Teile clonen:
    #--------------------------------------------------------------------------
    $self->SUPER::clone($clone);

    #--------------------------------------------------------------------------
    # Dann Kontainer kopieren:
    #--------------------------------------------------------------------------
    # (Hier müssen auch alle Elemente des Arrays gecloned werden, da das ja
    # auch Referenzen sind!)
    #--------------------------------------------------------------------------
    my @hist;
    for (@{$self->_hist()}) {
        my $h = $_->clone();
        push @hist, $h;
    }
    #$clone->_hist(\@hist);        # Referenz auf Kopie ablegen
    $clone->_hist([ @hist ]);        # Referenz auf Kopie ablegen


    return $clone;
} # sub clone


#------------------------------------------------------------------------------
# The getter/setter methods:
#------------------------------------------------------------------------------
sub max     { return shift->_property('max',     @_) }
sub _anz    { return shift->_property('_anz',    @_) }
sub _akt    { return shift->_property('_akt',    @_) }
sub _hist   { return shift->_property('_hist',   @_) }
sub _status { return shift->_property('_status', @_) }


#------------------------------------------------------------------------------
# Hilfsfunktionen für n und m:
#------------------------------------------------------------------------------
sub _inc_anz { $_[0]->_anz($_[0]->_anz()+1) }
sub _dec_anz { $_[0]->_anz($_[0]->_anz()-1) }
sub _inc_akt { $_[0]->_akt($_[0]->_akt()+1) }
sub _dec_akt { $_[0]->_akt($_[0]->_akt()-1) }


#==============================================================================
#
#
#   H I L F S M E T H O D E N
#
#   (nur diese [und clear] verwenden _hist)
#
#==============================================================================


sub _hist_eintrag {
    #--------------------------------------------------------------------------
    # Methode      : _ h i s t _ e i n t r a g
    # Klasse       : H i s t o r i e
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Setter / Getter für gespeichertes Objekt.
    #                Setter trägt ein Objekt an gegebener Position (nicht
    #                Index, Position beginnt bei 1) ein (wird gecloned).
    #                Getter holt Objekt und gibt Clone davon zurück!
    # Parameter    : 1) Nummer (nicht Index!)
    #                2) Eintrag (wird in der Funktion gecloned) (falls Setter)
    # Rückgabewert : Der geclonte aktuelle Wert (im Falle des Getters) oder
    #                undef.
    #--------------------------------------------------------------------------
    # 0.0.1 - 21.02.2004 - CD - erstellt
    #--------------------------------------------------------------------------

    die "Graph::GED::Historie::_hist_eintrag mit falscher Parameterzahl " .
        "aufgerufen!"
        if scalar @_ < 2 or scalar @_ > 3;

    print "Graph::GED::Historie::_hist_eintrag(Stelle: $_[1], ",
          (@_ == 3 ? 'S' : 'G'),
          "etter)\n"
          if $Debug>1;

    if (@_ == 3) {
        #print Dumper($_[2]);
        #die "kein gültiges Objekt!" unless $_[2]->isa('Graph::GED::Basis');
        die "kein gültiges Objekt!" unless $_[2]->can('clone');

        $_[0]->_hist()->[$_[1]-1] = $_[2]->clone();
        return undef;
    }
    else {
        die "Eintrag nicht vorhanden!" if $_[0]->_anz() < $_[1];
        return $_[0]->_hist()->[$_[1]-1]->clone();
    }

} # sub _hist_eintrag


sub _hist_loesch_ersten_eintrag {
    #--------------------------------------------------------------------------
    # Methode      : _ h i s t _ l o e s c h _ e r s t e n _ e i n t r a g
    # Klasse       : Historie
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Entfernt den ersten (ältesten) Eintrag aus dem Array und
    #                löscht diesen richtig mit dessen interner Methode clear().
    # Parameter    : keine
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 21.02.2004 - CD - erstellt
    # 0.0.2 - 08.12.2004 - CD - Fehler entfernt (es wurde bei can auf $_[2]
    #                           statt auf $_[0] zugegriffen.
    #                           $self statt $_[0] verwendet.
    #--------------------------------------------------------------------------

    my ($self) = @_;

    die "Graph::GED::Historie::_hist_loesch_ersten_eintrag mit Parametern " .
        "aufgerufen!"
        if scalar @_ != 1;

    print "Graph::GED::Historie::_hist_loesch_ersten_eintrag\n" if $Debug>1;

    die "keine Einträge zum Löschen da!" if $self->_anz() < 1;

    #use Data::Dumper;print Data::Dumper::Dumper(@_);
    die "kein gültiges Objekt!" unless $self->can('clear');

    shift( @{$self->_hist()} )->clear();

} # sub _hist_loesch_ersten_eintrag


sub _hist_loesch_eintrag {
    #--------------------------------------------------------------------------
    # Methode      : _ h i s t _ l o e s c h _ e i n t r a g
    # Klasse       : Historie
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Entfernt den angegebenen Eintrag (nicht Index, sondern
    #                mit 1 beginnende Stelle!) aus dem Array und löscht diesen
    #                richtig mit dessen interner Methode clear().
    # Parameter    : Stelle (beginnend mit 1) die gelöscht werden soll.
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 21.02.2004 - CD - erstellt
    #--------------------------------------------------------------------------

    die "Graph::GED::Historie::_hist_loesch_eintrag mit falscher Parameter" .
        "zahl aufgerufen!"
        if scalar @_ != 2;

    print "Graph::GED::Historie::_hist_loesch_eintrag(Stelle $_[1])\n"
          if $Debug>1;

    die "nicht genug Einträge zum Löschen da!" if $_[0]->_anz() < $_[1];

    my $del = splice @{$_[0]->_hist()}, $_[1]-1, 1;
    $del->clear();
    $del = undef; # unnötig

} # sub _hist_loesch_eintrag



#==============================================================================
#
#
#   M E T H O D E N
#
#
#==============================================================================


sub clear {
    #--------------------------------------------------------------------------
    # Methode      : c l e a r
    # Klasse       : Graph::GED::Historie
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Löscht alle Ecken und Kanten.
    # Parameter    : keine
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 09.02.2003 - CD - Erstellt
    # 0.0.2 - 10.02.2004 - CD - neue Hilfsmethoden verwendet.
    #                           Umstellung auf die Eckenklasse.
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Historie::clear mit Parametern aufgerufen!"
        if scalar @_ > 0;

    print "Graph::GED::Historie::clear\n" if $Debug;

    #--------------------------------------------------------------------------
    # Arraydaten löschen:
    #--------------------------------------------------------------------------
    $self->_hist(undef);
    $self->_hist([]);

    #--------------------------------------------------------------------------
    # Basisfunktionalität aufrufen:
    #--------------------------------------------------------------------------
    $self->SUPER::clear();

} # sub Graph::GED::Historie::clear


sub objekt_eintragen {
    #--------------------------------------------------------------------------
    # Methode      : o b j e k t _ e i n t r a g e n
    # Klasse       : Graph::GED::Historie
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Mit dieser Funktion möchte man den aktuellen Zustand des
    #                übergebenen Objekt in die Hitorie kopieren.
    # Parameter    : Objekt
    # Rückgabewert : (Anzahl älterer gespeicherter Einträge inklusive diesem,
    #                Anzahl neuere Einträge [bei dieser Funktion immer 0, da
    #                diese verworfen werden])
    #--------------------------------------------------------------------------
    # 0.0.1 - 12.02.2004 - CD - erstellt
    # 0.0.2 - 16.02.2004 - CD - Umstellung auf State-Machine
    #--------------------------------------------------------------------------

    my $self    = shift;

    die "Graph::GED::Historie::objekt_eintragen mit falscher Anzahl " .
        "Parametern aufgerufen!"
        if scalar @_ != 1;

    my $eintrag = shift;

    print "Graph::GED::Historie::objekt_eintragen($eintrag)\n" if $Debug;

    #--------------------------------------------------------------------------
    # Konsistenzprüfung:
    #--------------------------------------------------------------------------
    $self->_fehler('objekt_eintragen') unless $self->_konsistent();

    #--------------------------------------------------------------------------
    # Behandlung der unterschiedlichen Stati:
    #--------------------------------------------------------------------------
    if ($self->_status() eq NORMAL) {
        $self->_hist_eintrag($self->_akt()+1, $eintrag);
        $self->_inc_anz();
        $self->_inc_akt();
        $self->_status(NORMAL_ENDE) if $self->_anz() == $self->max();
    }

    elsif ($self->_status() eq NORMAL_ENDE) {
        $self->_hist_loesch_ersten_eintrag();
        $self->_hist_eintrag($self->_akt(), $eintrag);
    }

    elsif (
           $self->_status() eq NACH_RUECK        or
           $self->_status() eq NACH_RUECK_ANFANG or
           $self->_status() eq NACH_WIEDER
          )
    {
        $self->_inc_akt();
        $self->_loesche($self->_akt()+1, $self->_anz());
        $self->_anz($self->_akt());
        if ($self->_anz() == $self->max()) {
            $self->_status(NORMAL_ENDE);
        }
        else {
            $self->_status(NORMAL);
        }
    }

    elsif ($self->_status() eq NACH_WIEDER_ENDE) {
        $self->_inc_akt();
        $self->_status(NORMAL_ENDE);
    }


    #--------------------------------------------------------------------------
    # Variablen für die Rückgabe:
    #--------------------------------------------------------------------------
    my $alt = $self->_akt();
    my $neu = $self->_anz() - $self->_akt();

    #--------------------------------------------------------------------------
    # Kontrollausgabe:
    #--------------------------------------------------------------------------
    print "objekt_eintragen:\n",
          "\tStatus : ", $Stati{$self->_status()}, "\n",
          "\tAktuell: ", $self->_akt(), "\n",
          "\tAnzahl : ", $self->_anz(), "\n",
          "\tMax    : ", $self->max(), "\n",
          "\talt    : ", $alt, "\n",
          "\tneu    : ", $neu, "\n",
          "\tStelle : ", $self->_akt(), "\n",
          if $Debug;


    return ($alt, $neu);

} # sub objekt_eintragen


sub rueckgaengig {
    #--------------------------------------------------------------------------
    # Methode      : r u e c k g a e n g i g
    # Klasse       : Graph::GED::Historie
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Setzt den Zeiger auf das aktuelle Element einen tiefer
    #                und gibt dieses Element zurück.
    #                Damit das Objekt gespeicherrt werden kann, wenn es keine
    #                weiter in der Zukunft liegenden Versionen gibt. (In die-
    #                sem Fall ist die Rücknahme auch eine echte Änderung.)
    # Parameter    : Objekt
    # Rückgabewert : (Objekt,
    #                 Anzahl älterer gespeicherter Einträge als dieses,
    #                 Anzahl neuere Einträge)
    #--------------------------------------------------------------------------
    # 0.0.1 - 12.02.2004 - CD - erstellt
    # 0.0.2 - 16.02.2004 - CD - Umstellung auf State-Machine
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Historie::rueckgaengig mit Parametern aufgerufen!"
        if scalar @_ != 1;

    my $eintrag = shift;

    print "Graph::GED::Historie::rueckgaengig($eintrag)\n" if $Debug;

    die "kann nicht weiter zurueck" if $self->_akt() < 1;

    #--------------------------------------------------------------------------
    # Variablen für die Rückgabe:
    #--------------------------------------------------------------------------
    my $stelle = -1;

    #--------------------------------------------------------------------------
    # Konsistenzprüfung:
    #--------------------------------------------------------------------------
    $self->_fehler('rueckgaengig') unless $self->_konsistent();

    #--------------------------------------------------------------------------
    # Behandlung der unterschiedlichen Stati:
    #--------------------------------------------------------------------------
    if ($self->_status() eq NORMAL) {
        if ($self->_anz() > 0 and $self->_akt() > 0) {
            $stelle = $self->_akt();
            $self->_hist_eintrag($self->_akt()+1, $eintrag);
            $self->_dec_akt();
            $self->_inc_anz();
            if ($self->_akt() == 0) {
                $self->_status(NACH_RUECK_ANFANG);
            }
            else {
                $self->_status(NACH_RUECK);
            }
        }
        else {
            die "kann nicht weiter zurueck";
        }
    }

    elsif ($self->_status() eq NORMAL_ENDE) {
        $self->_dec_akt(); # weil alles sich verschiebt
        $stelle = $self->_akt();
        $self->_hist_loesch_ersten_eintrag();
        $self->_hist_eintrag($self->_akt()+1, $eintrag);
        $self->_dec_akt(); # weil man zurückgeht
        $self->_status(NACH_RUECK);
    }

    elsif ($self->_status() eq NACH_RUECK) {
        $stelle = $self->_akt();
        $self->_dec_akt();
        $self->_status(NACH_RUECK_ANFANG) if $self->_akt() == 0;
    }

    elsif ($self->_status() eq NACH_RUECK_ANFANG) {
        die "Kann im Status NACH_RUECK_ANFANG nicht zurück.";
    }

    elsif ($self->_status() eq NACH_WIEDER) {
        $stelle = $self->_akt();
        $self->_dec_akt();
        if ($self->_akt() == 0) {
            $self->_status(NACH_RUECK_ANFANG);
        }
        else {
            $self->_status(NACH_RUECK);
        }
    }

    elsif ($self->_status() eq NACH_WIEDER_ENDE) {
        $stelle = $self->_akt();
        $self->_dec_akt();
        if ($self->_akt() == 0) {
            $self->_status(NACH_RUECK_ANFANG);
        }
        else {
            $self->_status(NACH_RUECK);
        }
    }


    #--------------------------------------------------------------------------
    # Weitere Variablen für die Rückgabe:
    #--------------------------------------------------------------------------
    my $alt = $self->_akt();
    my $neu = $self->_anz() - $self->_akt() - 1;

    #--------------------------------------------------------------------------
    # Kontrollausgabe:
    #--------------------------------------------------------------------------
    print "rueckgaengig:\n",
          "\tStatus : ", $Stati{$self->_status()}, "\n",
          "\tAktuell: ", $self->_akt(), "\n",
          "\tAnzahl : ", $self->_anz(), "\n",
          "\tMax    : ", $self->max(), "\n",
          "\talt    : ", $alt, "\n",
          "\tneu    : ", $neu, "\n",
          "\tStelle : ", $stelle, "\n",
          if $Debug;


    #my $ausgabe = @{$self->_hist()}[$stelle-1]->clone();
    my $ausgabe = $self->_hist_eintrag($stelle);

    return ($ausgabe, $alt, $neu);

} # sub rueckgaengig


sub wiederholung {
    #--------------------------------------------------------------------------
    # Methode      : w i e d e r h o l u n g
    # Klasse       : Graph::GED::Historie
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Setzt den Zeiger auf das aktuelle Element einen höher und
    #                gibt dieses Element zurück.
    # Parameter    : keine
    # Rückgabewert : (Objekt,
    #                 Anzahl älterer gespeicherter Einträge als dieses,
    #                 Anzahl neuere Einträge)
    #--------------------------------------------------------------------------
    # 0.0.1 - 12.02.2004 - CD - erstellt
    # 0.0.2 - 16.02.2004 - CD - Umstellung auf State-Machine
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Historie::wiederholung mit Parametern aufgerufen!"
        if scalar @_ > 0;

    print "Graph::GED::Historie::wiederholung\n" if $Debug;

    die "kann nicht weiter vor" if $self->_akt() >= $self->_anz();

    #--------------------------------------------------------------------------
    # Variablen für die Rückgabe:
    #--------------------------------------------------------------------------
    my $stelle = -1;

    #--------------------------------------------------------------------------
    # Konsistenzprüfung:
    #--------------------------------------------------------------------------
    $self->_fehler('wiederholung') unless $self->_konsistent();

    #--------------------------------------------------------------------------
    # Behandlung der unterschiedlichen Stati:
    #--------------------------------------------------------------------------
    if ($self->_status() eq NORMAL) {
        die "Kann im Status NORMAL nicht wiederholen.";
    }

    elsif ($self->_status() eq NORMAL_ENDE) {
        die "Kann im Status NORMAL_ENDE nicht wiederholen.";
    }

    elsif (
           $self->_status() eq NACH_RUECK        or
           $self->_status() eq NACH_RUECK_ANFANG or
           $self->_status() eq NACH_WIEDER
          )
    {
        $self->_inc_akt();
        $stelle = $self->_akt()+1;
        if ($self->_akt() == $self->_anz() - 1) {
            $self->_status(NACH_WIEDER_ENDE);
        }
        else {
            $self->_status(NACH_WIEDER);
        }
    }

    elsif ($self->_status() eq NACH_WIEDER_ENDE) {
        die "Kann im Status NACH_WIEDER_ENDE nicht wiederholen.";
    }



    #--------------------------------------------------------------------------
    # Weitere Variablen für die Rückgabe:
    #--------------------------------------------------------------------------
    my $alt = $self->_akt();
    my $neu = $self->_anz() - $self->_akt() - 1;

    #--------------------------------------------------------------------------
    # Kontrollausgabe:
    #--------------------------------------------------------------------------
    print "wiederholung:\n",
          "\tStatus : ", $Stati{$self->_status()}, "\n",
          "\tAktuell: ", $self->_akt(), "\n",
          "\tAnzahl : ", $self->_anz(), "\n",
          "\tMax    : ", $self->max(), "\n",
          "\talt    : ", $alt, "\n",
          "\tneu    : ", $neu, "\n",
          "\tStelle : ", $stelle, "\n",
          if $Debug;


    #my $ausgabe = @{$self->_hist()}[$stelle-1]->clone();
    my $ausgabe = $self->_hist_eintrag($stelle);


    return ($ausgabe, $alt, $neu);
} # sub wiederholung


sub _loesche {
    #--------------------------------------------------------------------------
    # Methode      : _ l o e s c h e
    # Klasse       : Graph::GED::Historie
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Löscht Einträge aus dem Array.
    # Parameter    : von, bis (beginnend bei 1)
    # Rückgabewert : keine
    #--------------------------------------------------------------------------
    # 0.0.1 - 16.02.2004 - CD - erstellt
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Historie::_loesche mit falscher Anzahl " .
        "Parametern aufgerufen!"
        if scalar @_ != 2;

    my $von = shift;
    my $bis = shift;

    print "Graph::GED::Historie::_loesche($von,$bis)\n" if $Debug;


    #--------------------------------------------------------------------------
    # Objekte rauswerfen:
    #--------------------------------------------------------------------------
    for (reverse $von..$bis) {
        $self->_hist_loesch_eintrag($_);
    }
} # sub _loesche


sub _konsistent {
    #--------------------------------------------------------------------------
    # Methode      : _ k o n s i s t e n t
    # Klasse       : Graph::GED::Historie
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Testet das Historienobjekt abhängig vom Status auf
    #                Konsistenz.
    # Parameter    : Objekt
    # Rückgabewert : 1 oder 0
    #--------------------------------------------------------------------------
    # 0.0.1 - 15.02.2004 - CD - erstellt
    # 0.0.2 - 08.12.2004 - CD - Prüfung für NORMAL_ENDE verbessert.
    #--------------------------------------------------------------------------

    my $self = shift;
    my $ret  = 1;


    if (
        $self->max()  < 3             or
        $self->_anz() > $self->max()  or
        $self->_akt() > $self->_anz() or
        $self->_akt() < 0
       )
    {
        $ret = 0;
#print "a\n";
    }

    elsif ($self->_status() eq NORMAL) {
        if (
            $self->_akt() != $self->_anz() or
            $self->_anz() >= $self->max()
           )
        {
            $ret = 0;
#print "b\n";
        }
    }

    elsif ($self->_status() eq NORMAL_ENDE) {
        if (
            $self->_akt() != $self->_anz() #or
            #$self->_anz() != $self->max()
            # bei NORMAL_ENDE muss Anzahl nicht gleich max sein. (gerade wenn
            # max sehr groß ist....)
           )
        {
            $ret = 0;
#print "c\n";
        }
    }

    elsif ($self->_status() eq NACH_RUECK) {
        if (
            $self->_akt() >= $self->_anz() or
            $self->_anz() >  $self->max()
           )
        {
            $ret = 0;
#print "d\n";
        }
    }

    elsif ($self->_status() eq NACH_RUECK_ANFANG) {
        if (
            $self->_akt() != 0 or
            $self->_anz() <= 0 or
            $self->_anz() >  $self->max()
           )
        {
            $ret = 0;
#print "e\n";
        }
    }

    elsif ($self->_status() eq NACH_WIEDER) {
        if ($self->_akt() == 0) {
            $ret = 0;
#print "f\n";
        }
    }

    elsif ($self->_status() eq NACH_WIEDER_ENDE) {
        if ($self->_akt() != $self->_anz() - 1) {
            $ret = 0;
#print "g\n";
        }
    }

    else {
        $ret = 0;
#print "h\n";
    }


    return $ret;

} # sub _konsistent


sub _fehler {
    #--------------------------------------------------------------------------
    # Methode      : _ f e h l e r
    # Klasse       : Graph::GED::Historie
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Aussagekräftiger Abbruch.
    # Parameter    : Methode von der aus _fehler aufgerufen wurde
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 16.02.2004 - CD - erstellt
    #--------------------------------------------------------------------------

    my $self = shift;

    die "Graph::GED::Historie::_fehler mit falscher Anzahl " .
        "Parametern aufgerufen!"
        if scalar @_ != 1;

    my $methode = shift;

    print "Graph::GED::Historie::_fehler($methode)\n" if $Debug;

    my $msg = 'Fehler in Methode ' . $methode . ":\n" .
              "\tStatus : " . $Stati{$self->_status()} . "\n" .
              "\tAktuell: " . $self->_akt()            . "\n" .
              "\tAnzahl : " . $self->_anz()            . "\n" .
              "\tMax    : " . $self->max()             . "\n";
              #Dumper($self);


    die $msg;

} # sub _fehler


1;

__END__

=head1 NAME


Graph::GED::Historie.pm - a module for take back and move ahead steps


=head1 COPYRIGHT

Copyright 2004-2005 Christian Duehl. All rights reserved.

This library is free software. You can redistribute it and/or
modify it under the same terms as perl itself.

=cut
