package de.duehl.math.geometry;

/*
 * Copyright 2016 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

/**
 * Diese Klasse steht für eine Linie zwischen zwei zweidimensionalen Punkten.
 *
 * @version 1.01     2012-10-30
 * @author Christian Dühl
 */

public class Line {

    /** Startpunkt der Linie */
    private Point start;

    /** Endpunkt der Linie */
    private Point end;

    /** Konstruktor */
    public Line() {
        this(new Point(), new Point());
    }

    /**
     * Konstruktor
     *
     * @param start
     *            Startpunkt der Linie
     * @param end
     *            Endpunkt der Linie
     */
    public Line(Point start, Point end) {
        this.start = start;
        this.end = end;
    }

    /**
     * Konstruktor
     *
     * @param that
     *            Andere Linie
     */
    public Line(Line that) {
        this.start = new Point(that.start);
        this.end = new Point(that.end);
    }

    public Point getStart() {
        return start;
    }

    public void setFrom(Point from) {
        this.start = from;
    }

    public Point getEnd() {
        return end;
    }

    public void setTo(Point to) {
        this.end = to;
    }

    /** Vertauscht Start- und Endpunkt. */
    public void swap() {
        Point help = start;
        start = end;
        end = help;
    }

    /**
     * Berechnet den Abstand der beiden Punkte.
     *
     * @return Abstand der beiden Punkte
     */
    public double getDistance() {
        double dX = getXDistance();
        double dY = getYDistance();
        double distance = Math.sqrt(dX * dX + dY * dY);
        return distance;
    }

    /**
     * Berechnet den Abstand der x-Koordinaten der beiden Punkte.
     *
     * @return Abstand der x-Koordinaten der beiden Punkte
     */
    public double getXDistance() {
        double startX = start.getX();
        double endX = end.getX();
        double distanceX = Math.abs(startX - endX);
        return distanceX;
    }

    /**
     * Berechnet den Abstand der y-Koordinaten der beiden Punkte.
     *
     * @return Abstand der y-Koordinaten der beiden Punkte
     */
    public double getYDistance() {
        double startY = start.getY();
        double endY = end.getY();
        double distanceY = Math.abs(startY - endY);
        return distanceY;
    }

    /**
     * Gibt an, ob die Linie senkrecht verläuft.
     *
     * @return Wahrheitswert
     */
    public boolean isVertical() {
        return getXDistance() < Geometry.EPSILON;
    }

    /**
     * Gibt an, ob die Linie horizontal verläuft.
     *
     * @return Wahrheitswert
     */
    public boolean isHorizontal() {
        return getYDistance() < Geometry.EPSILON;
    }


    /* --- generierte Methoden: --- */


    @Override
    public String toString() {
        return "Line [start=" + start + ", end=" + end + "]";
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((start == null) ? 0 : start.hashCode());
        result = prime * result + ((end == null) ? 0 : end.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        Line other = (Line) obj;
        if (start == null) {
            if (other.start != null) {
                return false;
            }
        }
        else if (!start.equals(other.start)) {
            return false;
        }
        if (end == null) {
            if (other.end != null) {
                return false;
            }
        }
        else if (!end.equals(other.end)) {
            return false;
        }
        return true;
    }

}
