package de.duehl.math.geometry.discrete;

/*
 * Copyright 2018 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import de.duehl.algorithm.backtracking.Backtracking;
import de.duehl.basics.text.NumberString;

/**
 * Diese Klasse erstellt eine endliche projektive Ebene mittels Backtracking.
 *
 * Angelehnt an das Spiel Dobble wird jede Gerade durch ein Symbol repräsentiert. Jeder
 * Punkt entspricht dann einer Spielkarte mit den acht Symbolen (Ordnung 7 + 1).
 *
 * @version 1.01     2018-01-02
 * @author Christian Dühl
 */

public class CreateFiniteProjectivePlane {

    /** Ordnung der Endlichen Projektiven Ebene. */
    private final int order;

    /** Endliche Projektive Ebene die hier Erzeugt wird. */
    private final FiniteProjectivePlane plane;

    /** Algorithmus zur Erzeugung der Endlichen Projektiven Ebene. */
    private final Backtracking<FiniteProjectivePlane, FiniteProjectivePlaneCreatingStep> backtracking;

    /**
     * Konstruktor.
     *
     * @param order
     *            Ordnung der Endlichen Projektiven Ebene.
     */
    public CreateFiniteProjectivePlane(int order) {
        this.order = order;

        plane = new FiniteProjectivePlane(order);

        backtracking = new Backtracking<>();
        backtracking.informAfterXSteps(100000);
    }

    /** Erzeugt die endliche projektive Ebene. */
    public void create() {
        System.out.println("Erstellung endlicher projektiver Ebene der Ordnung " + order + " läuft...");

        boolean hasSolution = backtracking.findSolution(plane);

        System.out.println("Anzahl rekursiver Aufrufe      : "
                + NumberString.taupu(backtracking.getRecursiveCount()));
        System.out.println("Anzahl versuchter Schritte     : "
                + NumberString.taupu(backtracking.getTriedStepsCount()));
        System.out.println("Anzahl zurückgenommene Schritte: "
                + NumberString.taupu(backtracking.getBacktrackingCount()));
        System.out.println("Laufzeit                       : " + backtracking.getRuntime());

        if (hasSolution) {
            System.out.println("Lösung:\n" + plane.toString());
        }
        else {
            System.out.println("Es gibt keine Lösung.");
        }
    }

    public static void main(String[] args) {
        new CreateFiniteProjectivePlane(3).create();
    }

}
