package de.duehl.math.geometry.discrete;

/*
 * Copyright 2018 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.util.ArrayList;
import java.util.List;

import de.duehl.algorithm.backtracking.AlgorithmState;
import de.duehl.basics.debug.Assure;

/**
 * Diese Klasse stellt eine endliche projektive Ebene dar.
 * Außerdem enthält sie die Methoden, die während des Backtracking-Algorithmus zum Aufbau der
 * Ebene benötigt werden.
 *
 * Angelehnt an das Spiel Dobble wird jede Gerade durch ein Symbol repräsentiert. Jeder
 * Punkt entspricht dann einer Spielkarte mit den acht Symbolen (Ordnung 7 + 1).
 *
 * @version 1.01     2018-01-02
 * @author Christian Dühl
 */

public class FiniteProjectivePlane implements AlgorithmState<FiniteProjectivePlaneCreatingStep> {

    private static final boolean DEBUG = false;

    /** Ordnung der Endlichen Projektiven Ebene. */
    private final int order;

    /** Die Geraden der endlichen projektiven Ebene. */
    private final List<FiniteProjectiveLine> lines;

    /** Die Punkte der endlichen projektiven Ebene. */
    private final List<FiniteProjectivePoint> points;

    /** Gibt an, ob beim letzten Ermitteln des nächsten Schritts ein solcher gefunden wurde. */
    private boolean chooseNextStepWasSuccessfull;

    /**
     * Konstruktor.
     *
     * @param order
     *            Ordnung der Endlichen Projektiven Ebene.
     */
    public FiniteProjectivePlane(int order) {
        this.order = order;

        lines = new ArrayList<>();
        points = new ArrayList<>();

        say("Projektive Ebene der Ordnung " + order);
        say("Anzahl Punkte               : " + numberOfPoints());
        say("Anzahl Geraden (und Symbole): " + numberOfLines());

        createLines();
        createPoints();
    }

    private void createLines() {
        List<Symbol> symbols = Symbol.createSymbols(numberOfLines());
        Assure.isEqual(symbols.size(), numberOfLines());

        for (Symbol symbol : symbols) {
            FiniteProjectiveLine line = new FiniteProjectiveLine(symbol, order);
            lines.add(line);
        }
    }

    private void createPoints() {
        for (int i = 0; i < numberOfPoints(); ++i) {
            String name = String.format("Punkt %s", i + 1);
            FiniteProjectivePoint point = new FiniteProjectivePoint(name, order);
            points.add(point);
        }
    }

    /** Gibt an, ob der Zustand während des Algorithmus zum Aufbau der Ebene (lokal) zulässig ist. */
    @Override
    public boolean isVaild() {
        /* Jede Gerade hat maximal n + 1 Punkte. */
        if (countMaximalNumberOfPointsOnTheLines() > numberOfPointsOnEachLine()) {
            return false;
        }

        /* Jeder Punkt liegt auf maximal n + 1 Geraden. */
        if (countMaximumNumberOfLinesThroughThePoints() > numberOfPointsOnEachLine()) {
            return false;
        }

        /* Jeder Punkt liegt höchstens einmal auf jeder Geraden. */
        for (FiniteProjectivePoint point : points) {
            if (!point.isValid()) {
                return false;
            }
        }

        /* Je zwei Punkte verbindet höchstens eine Gerade. */
        if (countMaximumNumberOfLinesBetweenTwoPoints() > 1) {
            return false;
        }

        /* komische dritte Bedingung...? */
        // TODO ...

        return true;
    }

    private int countMaximalNumberOfPointsOnTheLines() {
        int maximumCount = 0;

        for (FiniteProjectiveLine line : lines) {
            int count = 0;
            for (FiniteProjectivePoint point : points) {
                if (line.containesPoint(point)) {
                    ++count;
                }
            }
            if (count > maximumCount) {
                maximumCount = count;
            }
        }

        return maximumCount;
    }

    private int countMinimalNumberOfPointsOnTheLines() {
        int minimumCount = Integer.MAX_VALUE;

        for (FiniteProjectiveLine line : lines) {
            int count = 0;
            for (FiniteProjectivePoint point : points) {
                if (line.containesPoint(point)) {
                    ++count;
                }
            }
            if (count < minimumCount) {
                minimumCount = count;
            }
        }

        return minimumCount;
    }

    private int countMaximumNumberOfLinesThroughThePoints() {
        int maximumCount = 0;

        for (FiniteProjectivePoint point : points) {
            int count = 0;
            for (FiniteProjectiveLine line : lines) {
                if (point.liesOnLine(line)) {
                    ++count;
                }
            }
            if (count > maximumCount) {
                maximumCount = count;
            }
        }

        return maximumCount;
    }

    private int countMinimumNumberOfLinesThroughThePoints() {
        int minimumCount = Integer.MAX_VALUE;

        for (FiniteProjectivePoint point : points) {
            int count = 0;
            for (FiniteProjectiveLine line : lines) {
                if (point.liesOnLine(line)) {
                    ++count;
                }
            }
            if (count < minimumCount) {
                minimumCount = count;
            }
        }

        return minimumCount;
    }

    private int countMaximumNumberOfLinesBetweenTwoPoints() {
        int maximumCount = 0;

        for (int i = 0; i < points.size() - 1; ++i) {
            FiniteProjectivePoint p = points.get(i);
            List<FiniteProjectiveLine> linesIncidentWithP = p.getIncidentLines();
            for (int j = i + 1; j < points.size(); ++j) {
                FiniteProjectivePoint q = points.get(j);
                List<FiniteProjectiveLine> linesIncidentWithQ = q.getIncidentLines();
                int count = 0;
                for (FiniteProjectiveLine lineP : linesIncidentWithP) {
                    for (FiniteProjectiveLine lineQ : linesIncidentWithQ) {
                        if (lineP.equals(lineQ)) {
                            ++count;
                        }
                    }
                }
                if (count > maximumCount) {
                    maximumCount = count;
                }
            }
        }

        return maximumCount;
    }

    private int countMinimumNumberOfLinesBetweenTwoPoints() {
        int minimumCount = Integer.MAX_VALUE;

        for (int i = 0; i < points.size() - 1; ++i) {
            FiniteProjectivePoint p = points.get(i);
            List<FiniteProjectiveLine> linesIncidentWithP = p.getIncidentLines();
            for (int j = i + 1; j < points.size(); ++j) {
                FiniteProjectivePoint q = points.get(j);
                List<FiniteProjectiveLine> linesIncidentWithQ = q.getIncidentLines();
                int count = 0;
                for (FiniteProjectiveLine lineP : linesIncidentWithP) {
                    for (FiniteProjectiveLine lineQ : linesIncidentWithQ) {
                        if (lineP.equals(lineQ)) {
                            ++count;
                        }
                    }
                }
                if (count < minimumCount) {
                    minimumCount = count;
                }
            }
        }

        return minimumCount;
    }

    @Override
    public boolean isComplete() {
        /* Jede Gerade hat genau n + 1 Punkte. */
        if (countMaximalNumberOfPointsOnTheLines() != numberOfPointsOnEachLine()) {
            return false;
        }
        if (countMinimalNumberOfPointsOnTheLines() != numberOfPointsOnEachLine()) {
            return false;
        }

        /* Jeder Punkt liegt auf genau n + 1 Geraden. */
        if (countMaximumNumberOfLinesThroughThePoints() != numberOfPointsOnEachLine()) {
            return false;
        }
        if (countMinimumNumberOfLinesThroughThePoints() != numberOfPointsOnEachLine()) {
            return false;
        }

        /* Jeder Punkt liegt genau einmal auf jeder Geraden. */
        for (FiniteProjectivePoint point : points) {
            if (!point.isValid()) {
                return false;
            }
            if (!point.isComplete()) {
                return false;
            }
        }

        /* Je zwei Punkte verbindet höchstens eine Gerade. */
        if (countMaximumNumberOfLinesBetweenTwoPoints() != 1) {
            return false;
        }
        if (countMinimumNumberOfLinesBetweenTwoPoints() != 1) {
            return false;
        }

        /* komische dritte Bedingung...? */
        // TODO ...

        return true;
    }

    /** Teillösungsschritt ausführen. */
    @Override
    public void forward(FiniteProjectivePlaneCreatingStep step) {
        FiniteProjectiveLine line = step.getLine();
        FiniteProjectivePoint point = step.getPoint();

        say("    step: füge incidente Gerade '" + line.getSymbol() + "' zu " + point + " hinzu");
        point.addIncidentLine(line);
        line.addIncidentPoint(point);
    }

    /** Backtracking. */
    @Override
    public void backward(FiniteProjectivePlaneCreatingStep step) {
        FiniteProjectiveLine line = step.getLine();
        FiniteProjectivePoint point = step.getPoint();

        say("backstep: entferne incidente Gerade '" + line.getSymbol() + "' aus " + point);
        point.removeIncidentLine(line);
        line.removeIncidentPoint(point);
    }

    /**
     * Wählt den nächsten Teilschritt aus, wobei die Liste der angegebenen Teilschritte sich
     * bereits als Falsch erwiesen haben.
     */
    @Override
    public FiniteProjectivePlaneCreatingStep chooseNextStep(
            List<FiniteProjectivePlaneCreatingStep> badSteps) {
        for (FiniteProjectiveLine line : lines) {
            if (!line.isMaximal()) {
                for (FiniteProjectivePoint point : points) {
                    if (!point.isMaximal() && !line.containesPoint(point)) {
                        FiniteProjectivePlaneCreatingStep step =
                                new FiniteProjectivePlaneCreatingStep(line, point);
                        if (!badSteps.contains(step)) {
                            chooseNextStepWasSuccessfull = true;
                            say("Nächster Schritt: " + step);
                            return step;
                        }
                    }
                }
            }
        }

        chooseNextStepWasSuccessfull = false;
        return null;
    }

    /** Gibt an, ob bei der Auswahl des nächsten Schritts ein solcher erzeugt werden konnte. */
    @Override
    public boolean chooseNextStepWasSuccessfull() {
        return chooseNextStepWasSuccessfull;
    }


//    private void distributeSymbolsAtPoints() {
//        while (countMinimalSymbolsOnAllPoints() < numberOfLinesThroughEachPoint()) {
//            for (Symbol symbol : symbols) {
//                distributeSymbolAtPoints(symbol);
//            }
//        }
//    }

//    private int countMinimalSymbolsOnAllPoints() {
//        int minimal = Integer.MAX_VALUE;
//
//        for (Symbol symbol : symbols) {
//            int count = countSymbolOnAllPoints(symbol);
//            if (count < minimal) {
//                minimal = count;
//            }
//        }
//
//        return minimal;
//    }

//    private void distributeSymbolAtPoints(Symbol symbol) {
//        while (countSymbolOnAllPoints(symbol) < numberOfLinesThroughEachPoint()) {
//            FiniteProjectivePoint point = determineFirstPointWithoutSymbol(symbol);
//            if (point == NO_POINT_FOUND) {
//                backtrace();
//            }
//            else {
//                addStep(symbol, point);
//            }
//        }
//    }

//    private int countSymbolOnAllPoints(Symbol symbol) {
//        int count = 0;
//
//        for (FiniteProjectivePoint point : points) {
//            if (point.containsSymbol(symbol)) {
//                ++count;
//            }
//        }
//
//        return count;
//    }

//    private void backtrace() {
//        if (backtracking.isEmpty()) {
//            SystemTools.sleep(50);
//            System.out.flush();
//            System.err.flush();
//            String message = "Kein weiteres Backtracking möglich";
//            System.err.println(message + ". Punkte:");
//            printPoints();
//            System.err.println(message);
//            throw new RuntimeException(message);
//        }
//
//        PlaneCreatingStep lastStep = backtracking.backtrack();
//        points = lastStep.getPoints();
//        FiniteProjectivePoint point = lastStep.getLastChangedPoint();
//        Symbol symbol = lastStep.getLastAddedSymbol();
//        say("backstep: entferne Symbol '" + symbol + "' aus " + point);
//        point.forbidSymbol(symbol);
//    }

//    private void addStep(Symbol symbol, FiniteProjectivePoint point) {
//        say("step    : füge Symbol '" + symbol + "' zu " + point + " hinzu.");
//        point.addSymbol(symbol);
//        backtracking.addState(new PlaneCreatingStep(point, symbol, points));
//    }
//
//    private FiniteProjectivePoint determineFirstPointWithoutSymbol(Symbol symbol) {
//        for (FiniteProjectivePoint point : points) {
//            if (!point.containsSymbol(symbol)
//                    && !point.hasFullNumberOfSymbols()
//                    && !point.isForbiddenSymbol(symbol)
//                    && !otherPointHasSymbolPair(point, symbol)) {
//                return point;
//            }
//        }
//        return NO_POINT_FOUND;
//    }
//
//    private boolean otherPointHasSymbolPair(FiniteProjectivePoint point, Symbol symbol) {
//        for (Symbol symbolOnPoint : point.getSymbols()) {
//            for (FiniteProjectivePoint otherPoint : points) {
//                if (!point.equals(otherPoint)
//                        && otherPoint.containsSymbol(symbolOnPoint)
//                        && otherPoint.containsSymbol(symbol)) {
//                    return true;
//                }
//            }
//        }
//
//        return false;
//    }

//    private void printPoints() {
//        System.out.println(toString());
//    }

    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder();
        for (FiniteProjectivePoint point : points) {
            builder.append(point);
            builder.append("\n");
        }
        return builder.toString();
    }

    /** Anzahl der Geraden in der endlichen projektiven Ebene. */
    public int numberOfLines() {
        return numberOfLines(order);
    }

    /** Anzahl der Punkte in der endlichen projektiven Ebene. */
    public int numberOfPoints() {
        return numberOfPoints(order);
    }

    /** Anzahl der Geraden durch jeden Punkt in der endlichen projektiven Ebene. */
    public int numberOfLinesThroughEachPoint() {
        return numberOfLinesThroughEachPoint(order);
    }

    /** Anzahl der Punkte auf jeder Geraden in der endlichen projektiven Ebene. */
    public int numberOfPointsOnEachLine() {
        return numberOfPointsOnEachLine(order);
    }

    /** Anzahl der Geraden in der endlichen projektiven Ebene der angegebenen Ordnung. */
    public static int numberOfLines(int order) {
        return order * order + order + 1;
    }

    /** Anzahl der Punkte in der endlichen projektiven Ebene der angegebenen Ordnung. */
    public static int numberOfPoints(int order) {
        return numberOfLines(order);
    }

    /**
     * Anzahl der Geraden durch jeden Punkt in der endlichen projektiven Ebene der angegebenen
     * Ordnung.
     */
    public static int numberOfLinesThroughEachPoint(int order) {
        return numberOfPointsOnEachLine(order);
    }

    /**
     * Anzahl der Punkte auf jeder Geraden in der endlichen projektiven Ebene der angegebenen
     * Ordnung.
     */
    public static int numberOfPointsOnEachLine(int order) {
        return order + 1;
    }

    private void say(Object object) {
        if (DEBUG) {
            System.out.println(object.toString());
        }
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((lines == null) ? 0 : lines.hashCode());
        result = prime * result + order;
        result = prime * result + ((points == null) ? 0 : points.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        FiniteProjectivePlane other = (FiniteProjectivePlane) obj;
        if (lines == null) {
            if (other.lines != null) {
                return false;
            }
        }
        else if (!lines.equals(other.lines)) {
            return false;
        }
        if (order != other.order) {
            return false;
        }
        if (points == null) {
            if (other.points != null) {
                return false;
            }
        }
        else if (!points.equals(other.points)) {
            return false;
        }
        return true;
    }

}
