package de.duehl.math.geometry.discrete;

/*
 * Copyright 2018 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.util.ArrayList;
import java.util.List;

/**
 * Diese Klasse stellt einen Punkt einer endlichen projektiven Ebene dar.
 *
 * @version 1.01     2018-01-02
 * @author Christian Dühl
 */

public class FiniteProjectivePoint {

    /** Name des Punktes. */
    private final String name;

    /** Liste der mit diesem Punkt incidenten Geraden. */
    private final List<FiniteProjectiveLine> incidentLines;

    /** Maximale Anzahl der Linien durch diesen Punkt. */
    private final int maximalNumberOfLines;

    /**
     * Konstruktor.
     *
     * @param name
     *            Name des Punktes.
     * @param order
     *            Ordnung der Endlichen Projektiven Ebene.
     */
    public FiniteProjectivePoint(String name, int order) {
        this.name = name;

        maximalNumberOfLines = FiniteProjectivePlane.numberOfLinesThroughEachPoint(order);
        incidentLines = new ArrayList<>();
    }

    /** Gibt an, ob der Punkt auf der Linie liegt. */
    public boolean liesOnLine(FiniteProjectiveLine line) {
        return incidentLines.contains(line);
    }

    /**
     * Es wird geprüft, ob der Punkt auf jeder Geraden höchstens einmal liegt.
     *
     * Man könnte auch sagen, ob die Symbole auf den Punkten disjunkt sind.
     * Zwei Geraden sind genau dann gleich, wenn sie das gleiche Symbol enthalten.
     */
    public boolean isValid() {
        for (int i = 0; i < incidentLines.size() - 2; ++i) {
            FiniteProjectiveLine lineI = incidentLines.get(i);
            for (int j = i + 1; j < incidentLines.size() - 1; ++j) {
                FiniteProjectiveLine lineJ = incidentLines.get(j);
                if (lineI.equals(lineJ)) {
                    return false;
                }
            }
        }

        return true;
    }

    /** Gibt an, ob der Punkt vollständig besetzt ist. */
    public boolean isMaximal() {
        return incidentLines.size() == maximalNumberOfLines;
    }

    /** Gibt an, ob der Punkt zulässig und vollständig ist. */
    public boolean isComplete() {
        return isValid() && isMaximal();
    }

    /** Getter für den Namen des Punktes. */
    public String getName() {
        return name;
    }

    /** Getter für die Liste der mit diesem Punkt incidenten Geraden. */
    public List<FiniteProjectiveLine> getIncidentLines() {
        return incidentLines;
    }

    /** Fügt eine incidente Gerade hinzu. */
    public void addIncidentLine(FiniteProjectiveLine line) {
        incidentLines.add(line);
    }

    /** Entfernt eine incidente Gerade. */
    public void removeIncidentLine(FiniteProjectiveLine line) {
        incidentLines.remove(line);
    }

    /** Stringrepresentation. */
    @Override
    public String toString() {
        return name + " " + getIncidentLineSymbols();
    }

    private List<Symbol> getIncidentLineSymbols() {
        List<Symbol> symbols = new ArrayList<>();
        for (FiniteProjectiveLine line : incidentLines) {
            symbols.add(line.getSymbol());
        }
        return symbols;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((name == null) ? 0 : name.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        FiniteProjectivePoint other = (FiniteProjectivePoint) obj;
        if (name == null) {
            if (other.name != null) {
                return false;
            }
        }
        else if (!name.equals(other.name)) {
            return false;
        }
        return true;
    }

}
