package de.duehl.math.sets;

/*
 * Copyright 2019 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.util.ArrayList;
import java.util.List;


/**
 * Diese Klasse erzeugt die Menge aller Kombinationen zu einer gegebene Liste von an dieser
 * Position jeweils zulässigen Elementen.
 *
 * @version 1.01     2019-04-16
 * @author Christian Dühl
 */

public class Combinations<Element> {

    /** Liste mit den möglichen Elementen an jeder Position. */
    private final List<List<Element>> possibleElementsAtEachPositionList;

    /**
     * Liste mit den Kombinationen, ein Element passt zu einem Element in der Reihenfolge der
     * Listen in possibleElementsAtEachPositionList.
     */
    private List<List<Element>> combinations;

    /**
     * Konstruktor.
     *
     * @param possibleElementsAtEachPositionList
     *            Liste mit den möglichen Elementen an jeder Position.
     */
    public Combinations(List<List<Element>> possibleElementsAtEachPositionList) {
        this.possibleElementsAtEachPositionList = possibleElementsAtEachPositionList;

        for (List<Element> possibleElements : possibleElementsAtEachPositionList) {
            if (possibleElements.isEmpty()) {
                throw new IllegalArgumentException("Keine der Elementlisten darf leer sein!");
            }
        }
    }

    /** Erstellt die Liste aller möglichen Kombinationen. */
    public List<List<Element>> createCombinations() {
        combinations = new ArrayList<>();

        for (List<Element> possibleElements : possibleElementsAtEachPositionList) {
            workOnCombination(possibleElements);
        }

        return combinations;
    }

    private void workOnCombination(List<Element> possibleElements) {
        if (combinations.isEmpty()) {
            for (Element possibleElement : possibleElements) {
                List<Element> newCombination = new ArrayList<>();
                newCombination.add(possibleElement);
                combinations.add(newCombination);
            }
        }
        else if (possibleElements.size() == 1) {
            for (List<Element> combination : combinations) {
                combination.add(possibleElements.get(0));
            }
        }
        else {
            int sizeBeforeMultiplication = combinations.size();
            multiplyExistingCominations(possibleElements.size());
            if (combinations.size() != sizeBeforeMultiplication * possibleElements.size()) {
                throw new RuntimeException("Interner Fehler.");
            }
            int index = -1;
            for (Element possibleElement : possibleElements) {
                for (int i = 0; i < sizeBeforeMultiplication; ++i) {
                    combinations.get(++index).add(possibleElement);
                }
            }
        }
    }

    private void multiplyExistingCominations(int factor) {
        List<List<Element>> allCopies = new ArrayList<>();
        for (int i = 0; i < factor - 1; ++i) {
            List<List<Element>> copied = copyListOfElementList(combinations);
            allCopies.addAll(copied);
        }
        combinations.addAll(allCopies);
    }

    private List<List<Element>> copyListOfElementList(List<List<Element>> source) {
        List<List<Element>> target = new ArrayList<>();

        for (List<Element> elementList : source) {
            List<Element> newElementList = copyElementList(elementList);
            target.add(newElementList);
        }

        return target;
    }

    private List<Element> copyElementList(List<Element> source) {
        List<Element> target = new ArrayList<>();

        for (Element element : source) {
            target.add(element);
        }

        return target;
    }

}
