package de.duehl.swing.ui.dialogs.html;

/*
 * Copyright 2018 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.awt.CardLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.io.IOException;
import java.net.URL;

import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;

import de.duehl.basics.text.Text;
import de.duehl.basics.text.html.HtmlTool;
import de.duehl.swing.ui.GuiTools;

/**
 * Diese Klasse stellt ein Panel zur Verfügung, das Html darstellt. Man kann es umschalten, um den
 * Quellcode zu sehen.
 *
 * @version 1.01     2018-03-27
 * @author Christian Dühl
 */

public class HtmlPanel {

    private static final Dimension PANEL_DIMENSION = new Dimension(1000, 750);

    /** Pane zur Darstellung des HTML. */
    private final JEditorPane ePane;

    /** ScrollPane der Pane zur Darstellung des HTML. */
    private final JScrollPane scroll;

    /** TextArea zur Darstellung des Sourcecodes. */
    private final JTextArea textArea;

    /** ScrollPane der TextArea zur Darstellung des Sourcecodes. */
    private final JScrollPane textScroll;

    /** Panel in dem HTML dargestellt wird. */
    private final JPanel htmlPanel;

    /** Das CardLayout, wird beim Weiterschalten benötigt. */
    private final CardLayout cardLayout;

    /** Der Panel mit dem CardLayout, wird beim Weiterschalten benötigt. */
    private final JPanel cardPanel;

    /** Diese Enum-Klasse repräsentiert die Karten im CardLayout. */
    private enum Card {
        HTML, SOURCE_CODE;

        public Card getNext() {
            int actualOrdinal = ordinal();
            int nextOrdinal = actualOrdinal + 1;
            if (nextOrdinal >= values().length) {
                nextOrdinal = 0;
            }
            return values()[nextOrdinal];
        }
    }

    /** Aktuell angezeigte Karte im CardLayout. */
    private Card currentCard;

    /** Konstruktor. */
    public HtmlPanel() {
        ePane = new JEditorPane();
        scroll = new JScrollPane(ePane);

        textArea = new JTextArea();
        textScroll = new JScrollPane(textArea);

        cardPanel = new JPanel();
        cardLayout = new CardLayout();

        htmlPanel = buildThePanel();
    }

    /** Erstellt den Panel. */
    private JPanel buildThePanel() {
        cardPanel.setLayout(cardLayout);

        cardPanel.add(createScrollableEPane(), Card.HTML.name());
        cardPanel.add(createTextArea(), Card.SOURCE_CODE.name());

        currentCard = Card.HTML;

        return cardPanel;
    }

    /** Schaltet zur nächsten Karte weiter. */
    public void switchCard() {
        currentCard = currentCard.getNext();
        cardLayout.show(cardPanel, currentCard.name());
        cardPanel.validate();
    }

    private Component createScrollableEPane() {
        ePane.setContentType("text/html");

        //ePane.setFocusable(false); // Verhindert Markieren und Editieren.

        ePane.setEditable(false);    // Verhindert nur das Editieren, man kann aber Text markieren
                                     // und per Copy und Paste entnehmen!

        scroll.setPreferredSize(PANEL_DIMENSION);
        return scroll;
    }

    private Component createTextArea() {
        textArea.setEditable(false);

        textScroll.setPreferredSize(PANEL_DIMENSION);
        return textScroll;
    }

    /**
     * Schreibt den Text in den Hauptteil des Dialogs und setzt den Textcursor auf den Anfang des
     * Dokuments.
     *
     * @param url
     *            Datei, aus der der Inhalt gelesen wird.
     */
    public void showHtml(URL url) {
        showHtmlWithoutSettingCaretPosition(url);
        ePane.setCaretPosition(0);
        textArea.setCaretPosition(0);
    }

    /**
     * Schreibt den Text in den Hauptteil des Dialogs und setzt den Textcursor NICHT auf den Anfang
     * des Dokuments. Ansonsten funktionieren Links auf Abschnitte im gleichen Dokument nicht,
     * er springt kurz darunter, dann wieder an den Anfang zurück!
     *
     * @param url
     *            Datei, aus der der Inhalt gelesen wird.
     */
    private void showHtmlWithoutSettingCaretPosition(URL url) {
        try {
            tryToShowHtmlWithoutSettingCaretPosition(url);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * Schreibt den Text in den Hauptteil des Dialogs.
     *
     * @param url
     *            URL aus der der Inhalt gelesen wird.
     * @throws IOException Wenn es beim Einlesen der URL Probleme gibt.
     */
    private void tryToShowHtmlWithoutSettingCaretPosition(URL url) throws IOException {
        ePane.setPage(url);
        ePane.addHyperlinkListener(createHyperLinkListener());

        textArea.setText(HtmlTool.createContentFromUrl(url));
    }

    /**
     * Erstellt einen HyperlinkListener, so dass man im HTML Links anklicken kann.
     *
     * Siehe http://www.java2s.com/Code/Java/Swing-JFC/DemonstratingtheHyperlinkListener.htm.
     */
    private HyperlinkListener createHyperLinkListener() {
        return new HyperlinkListener() {
            @Override
            public void hyperlinkUpdate(HyperlinkEvent event) {
                HyperlinkEvent.EventType typ = event.getEventType();

                if (typ == HyperlinkEvent.EventType.ACTIVATED) {
                    reactOnClickedLink(event.getURL());
                }
            }
        };
    }

    /** Reagiert auf den Klick auf einen Link. */
    private void reactOnClickedLink(URL url) {
        URL shownUrl = ePane.getPage();
        if (null == shownUrl) {
            showHtml(url);
        }
        else {
            /*
             * Da man auch von bla#Teil1 zu bla#Teil2 ohne Setzen der Caret-Position auskommen
             * möchte, werden vor dem Vergleich der URLs die Teile ab # abgeschnitten:
             */
            String urlString = Text.skipSublinkPart(url.toString());
            String shownUrlString = Text.skipSublinkPart(shownUrl.toString());
            if (urlString.equals(shownUrlString)) {
                //System.out.println("Zeige ohne SettingCaretPosition:\n\t"
                //        + "alte URL: " + shownUrl + "\n\t"
                //        + "neue URL: " + url);
                showHtmlWithoutSettingCaretPosition(url);
            }
            else {
                //System.out.println("Zeige mit SettingCaretPosition:\n\t"
                //        + "alte URL: " + shownUrl + "\n\t"
                //        + "neue URL: " + url);
                showHtml(url);
            }
        }
    }

    /**
     * Schreibt den Text in den Hauptteil des Dialogs.
     *
     * @param htmlContents
     *            Darzustellender Text.
     */
    public void showHtml(String htmlContents) {
        ePane.setText(htmlContents);
        ePane.setCaretPosition(0);

        textArea.setText(htmlContents);
        textArea.setCaretPosition(0);
    }

    /** Getter für den Panel in dem HTML dargestellt wird. */
    public JPanel getHtmlPanel() {
        return htmlPanel;
    }

    /** Scrollt zum Anfang. */
    public void setScrollPaneToMinimum() {
        GuiTools.setScrollPaneToMinimum(scroll);
        GuiTools.setScrollPaneToMinimum(textScroll);
    }

    /** Scrollt zum Ende. */
    public void setScrollPaneToMaximum() {
        GuiTools.setScrollPaneToMaximum(scroll);
        GuiTools.setScrollPaneToMaximum(textScroll);
    }

    /** Scrollt weiter nach oben. */
    public void setScrollPaneToPreviousSection() {
        GuiTools.setScrollPaneToPreviousSection(scroll);
        GuiTools.setScrollPaneToPreviousSection(textScroll);
    }

    /** Scrollt weiter nach unten. */
    public void setScrollPaneToNextSection() {
        GuiTools.setScrollPaneToNextSection(scroll);
        GuiTools.setScrollPaneToNextSection(textScroll);
    }

//    private void printScrollState() {
//        int minimum = scroll.getVerticalScrollBar().getMinimum();
//        int actualPos = scroll.getVerticalScrollBar().getValue();
//        int maximum = scroll.getVerticalScrollBar().getMaximum();
//
//        java.awt.Rectangle rect = scroll.getVisibleRect();
//        System.out.println("Position:\n"
//                + "\tMinimum    : " + minimum     + "\n"
//                + "\taktuell    : " + actualPos   + "\n"
//                + "\tMaximum    : " + maximum     + "\n"
//                + "\tRect x     : " + rect.x      + "\n"
//                + "\tRect y     : " + rect.y      + "\n"
//                + "\tRect width : " + rect.width  + "\n"
//                + "\tRect height: " + rect.height + "\n");
//    }

}
